import numpy as np


__all__ = ['isfloat', 'isint', 'isstr', 
           'res_doc_type', 'res_top_type', 'res_word_type']



# 
# Rudimentary type-checking fns
#


def isfloat(x):
    """
    Returns True if `x` is an instance of a float.
    """
    return (isinstance(x, np.inexact) or isinstance(x, np.float))


def isint(x):
    """
    Returns True if `x` is an instance of an int.
    """
    return (isinstance(x, np.integer) 
            or isinstance(x, int) or isinstance(x, long))


def isstr(x):
    """
    Returns True if `x` is an instance of a string.
    """
    return isinstance(x, basestring) or isinstance(x, np.flexible)


# 
# fns to resolve input polymorphism to the dist_*_* fns
#


def res_doc_type(corp, context_type, label_name, doc):
    """
    If `doc` is a string or a dict, performs a look up for its
    associated integer. If `doc` is a dict, looks for its label.
    Finally, if `doc` is an integer, stringifies `doc` for use as
    a label.
    
    Returns an integer, string pair: (<document index>, <document
    label>).
    """
    if isstr(doc):
        query = {label_name: doc}
        d = corp.meta_int(context_type, query)
    elif isinstance(doc, dict):
        d = corp.meta_int(context_type, doc)
        
        #TODO: Define an exception for failed queries in
        #vsm.corpus. Use it here.
        doc = corp.view_metadata(context_type)[label_name][d]
    else:
        d, doc = doc, str(doc)

    return d, doc
                    

def res_top_type(topic_or_topics):
    """
    If `topic_or_topics` is an int, then returns it in a list.
    """
    if isint(topic_or_topics):
        topic_or_topics = [topic_or_topics]

    return topic_or_topics


def res_word_type(corp, word):
    """
    If `word` is a string, performs a look up for its associated
    integer. Otherwise, stringifies `word`. 

    Returns an integer, string pair: (<word index>, <word label>).
    """
    if isstr(word):
        return corp.words_int[word], word

    return word, str(word)



import numpy as np
from vsm.corpus import Corpus


__all__ = ['eqva', 'new_material']


def eqva(a1, a2):
    """
    modified np.array_equal. considers a1 and a2
    equal when there is 1 difference.
    """
    a1.sort()
    a2.sort()
    count = 0
    a1_, a2_ = a1, a2
    if len(a1) > len(a2):
        a1_ = a2
        a2_ = a1

    for s in a1:
        if not s in a2:
            count += 1

    return count


def find_idx(ind, c1, c2):
    """
    finds exact match (1 diff) in c2 and returns the index.
    """
    ctx2 = c2.view_contexts('sentence', as_strings=True)
    ctx = c1.view_contexts('sentence', as_strings=True)[ind]
    
    for i in xrange(len(ctx2)):
        if eqva(ctx, ctx2[i]) < 2:
            return str(i)
    return ''


def new_material(c1, c2, idx=0):
    """
    Return new material in a list. 
    'idx' is an optional parameter for cutting off references.
    """
    ctx1 = c1.view_contexts('sentence', as_strings=True)
    
    if idx == 0:
        ctx2 = c2.view_contexts('sentence', as_strings=True)
    else:
        ctx2 = c2.view_contexts('sentence', as_strings=True)[:idx]
    len2 = len(ctx2)

    new = []
    for i in xrange(len(ctx1)):
        if i < len2:
            if len(ctx1[i]) == 0: # empty tokens.
                pass
            else:
                ind = find_idx(i, c1, c2)
                if len(ind) == 0:
                    new.append(i)
    return new



import numpy as np
import time
from vsm.split import split_corpus
from vsm.corpus import align_corpora as align
from ldafunctions import *
from _cgs_update import cgs_update
from progressbar import ProgressBar, Percentage, Bar


__all__ = [ 'LdaCgsSeq', 'LdaCgsQuerySampler' ]



class LdaCgsSeq(object):
    """
    An implementation of LDA using collapsed Gibbs sampling.
    """
    def __init__(self, corpus=None, context_type=None,
                 K=20, V=0, alpha=[], beta=[], seed=None):
        """
        Initialize LdaCgsSeq.

        :param corpus: Source of observed data.
        :type corpus: `Corpus`

        :param context_type: Name of tokenization stored in `corpus` whose tokens
            will be treated as documents.
        :type context_type: string, optional

        :param K: Number of topics. Default is `20`.
        :type K: int, optional

        :param alpha: Document priors. Default is a flat prior of 0.01
            for all topics.
        :type alpha: list, optional

        :param beta: Topic priors. Default is 0.01 for all words.
        :type beta: list, optional

        :param seed: Seed for numpy's RandomState. Default is `None`.
        :type seed: int, optional
        """

        self.context_type = context_type
        self.K = K

        if corpus:
            self.V = corpus.words.size
            self.indices = corpus.view_contexts(self.context_type, as_indices=True)
            self.corpus = corpus.corpus
        else:
            self.V = V
            self.indices = []
            self.corpus = []

        self.indices = np.array(self.indices, dtype='i')
        self.Z = np.zeros_like(self.corpus, dtype='i')

        priors = init_priors(self.V, self.K, beta, alpha)
        self.beta, self.alpha = priors

        self.word_top = (np.zeros((self.V, self.K), dtype=np.float)
                         + self.beta)
        if self.V==0:
            self.inv_top_sums = np.inf
        else:
            self.inv_top_sums = 1. / self.word_top.sum(0)
        self.top_doc = (np.zeros((self.K, len(self.indices)),
                                 dtype=np.float) + self.alpha)

        self.iteration = 0
        self.log_probs = []

        if seed is None:
            maxint = np.iinfo(np.int32).max
            self.seed = np.random.randint(0, maxint)
        else:
            self.seed = seed
        self._mtrand_state = np.random.RandomState(self.seed).get_state()


    @property
    def Z_split(self):
        return split_corpus(self.Z, self.indices)


    @property
    def docs(self):
        return split_corpus(self.corpus, self.indices)


    def _compute_word_top(self):
        self.word_top = compute_word_top(self.docs, self.Z_split, self.K,
                                         self.V, self.beta)


    def _compute_top_doc(self):
        self.top_doc = compute_top_doc(self.Z_split, self.K, self.alpha)


    def _compute_log_prob(self, increment=False):
        log_prob = compute_log_prob(self.docs, self.Z_split,
                                    self.word_top, self.top_doc)
        if increment:
            self.log_probs.append((self.iteration, log_prob))
            self.iteration += 1
        else:
            return log_prob


    def train(self, n_iterations=100, verbose=1, **kwargs):
        """
        Takes an optional argument, `n_iterations` and updates the model
        `n_iterations` times.

        :param n_iterations: Number of iterations. Default is 100.
        :type n_iterations: int, optional

        :param verbose: If 1, current number of iterations
            are printed out to notify the user. Default is 1.
        :type verbose: int, optional
        
        :param kwargs: For compatability with calls to LdaCgsMulti.
        :type kwargs: optional
        """
        random_state = np.random.RandomState(self.seed)
        random_state.set_state(self._mtrand_state)


        if verbose > 0:
            print ('Begin LDA training for {0} iterations'\
                   .format(n_iterations))
            start = time.time()
            t = start

        # Training loop
        stop = self.iteration + n_iterations
        if verbose == 1:
            pbar = ProgressBar(widgets=[Percentage(), Bar()], maxval=n_iterations).start()

        #print("Stop ", stop)
        for itr in xrange(self.iteration , stop):

            results = cgs_update(self.iteration, self.corpus, self.word_top,
                                 self.inv_top_sums, self.top_doc, self.Z,
                                 self.indices, self._mtrand_state[0],
                                 self._mtrand_state[1], self._mtrand_state[2],
                                 self._mtrand_state[3], self._mtrand_state[4])

            lp = results[4]
            self.log_probs.append((self.iteration, lp))

            if verbose == 2:
                itr_time = np.around(time.time()-t, decimals=1)
                t = time.time()
                if verbose > 1 or itr==stop-1:
                    print ('\nIteration {0} complete: log_prob={1}, time={2}'
                           .format(self.iteration, lp, itr_time))

            if verbose == 1:
                #print("Self iteration", self.iteration)
                pbar.update(self.iteration - (stop - n_iterations))
                time.sleep(0.01)

            self.iteration += 1

            self._mtrand_state = results[5:]
        if verbose == 1:
            pbar.finish();
        if verbose > 1:
            print '-'*60, ('\n\nWalltime per iteration: {0} seconds'
                           .format(np.around((t-start)/n_iterations, decimals=2)))


    @staticmethod
    def load(filename):
        """
        A static method for loading a saved LdaCgsMulti model.

        :param filename: Name of a saved model to be loaded.
        :type filename: string

        :returns: m : LdaCgsMulti object

        :See Also: :class:`numpy.load`
        """
        return load_lda(filename, LdaCgsSeq)


    def save(self, filename):
        """
        Saves the model in an `.npz` file.

        :param filename: Name of a saved model to be loaded.
        :type filename: string

        :See Also: :class:`numpy.savez`
        """
        save_lda(self, filename)



class LdaCgsQuerySampler(LdaCgsSeq):
    """
    """
    def __init__(self, lda_obj=None, new_corpus=None,
                 align_corpora=False, old_corpus=None,
                 context_type=None):

        if align_corpora:
            new_corp = align(old_corpus, new_corpus)

        if lda_obj:
            if context_type is None:
                context_type = lda_obj.context_type

            kwargs = dict(corpus=new_corpus,
                          context_type=context_type,
                          K=lda_obj.K, V=lda_obj.V,
                          alpha=lda_obj.alpha, beta=lda_obj.beta)
        else:
            kwargs = dict(corpus=new_corpus)


        super(LdaCgsQuerySampler, self).__init__(**kwargs)

        if lda_obj:
            self.word_top[:] = lda_obj.word_top
            self.inv_top_sums[:] = lda_obj.inv_top_sums




#################################################################
#                            Demos
#################################################################


def demo_LdaCgsSeq(doc_len=500, V=100000, n_docs=100,
                   K=20, n_iterations=5,
                   corpus_seed=None, model_seed=None):

    from vsm.extensions.corpusbuilders import random_corpus

    print 'Words per document:', doc_len
    print 'Words in vocabulary:', V
    print 'Documents in corpus:', n_docs
    print 'Number of topics:', K
    print 'Iterations:', n_iterations

    c = random_corpus(n_docs*doc_len, V, doc_len, doc_len+1, seed=corpus_seed)
    m = LdaCgsSeq(c, 'document', K=K, seed=model_seed)
    m.train(n_iterations=n_iterations, verbose=2)

    return m



import numpy as np
from vsm.corpus import Corpus, binary_search
from vsm.extensions.corpusbuilders import *
from vsm.extensions.corpusbuilders.util import *
from vsm.extensions.htrc import vol_link_fn, add_link_
import os
import re

__all__ = ['CorpusSent', 'sim_sent_sent', 'sim_sent_sent_across',
        'file_tokenize', 'file_corpus', 'dir_tokenize', 'dir_corpus',
        'extend_sdd', 'extend_across']


class CorpusSent(Corpus):
    """
    A subclass of Corpus whose purpose is to store original
    sentence information in the Corpus
    
    :See Also: :class: Corpus
    """
    def __init__(self, corpus, sentences, context_types=[], context_data=[], 
		remove_empty=False):
       
       super(CorpusSent, self).__init__(corpus, context_types=context_types,
		 context_data=context_data, remove_empty=remove_empty)
       
       sentences = [re.sub('\n', ' ', s) for s in sentences]
       self.sentences = np.array(sentences)


    def __set_words_int(self):
        """
        Mapping of words to their integer representations.
        """
        self.words_int = dict((t,i) for i,t in enumerate(self.words))

    def in_place_stoplist(self, stoplist=None, freq=0):
        """ 
        Changes a Corpus object with words in the stoplist removed and with 
        words of frequency <= `freq` removed.
        
        :param stoplist: The list of words to be removed.
        :type stoplist: list

        :param freq: A threshold where words of frequency <= 'freq' are
            removed. Default is 0.
        :type freq: integer, optional
            
        :returns: Copy of corpus with words in the stoplist and words
            of frequnecy <= 'freq' removed.

        :See Also: :class:`Corpus`
        """
        if stoplist is None:
            stoplist = list()
        else:
            # convert to raw list from set, array, etc.
            stoplist = [word for word in stoplist]

        if freq:
            #TODO: Use the TF model instead

            # print 'Computing collection frequencies'
            cfs = np.zeros_like(self.words, dtype=self.corpus.dtype)
    
            for word in self.corpus:
                cfs[word] += 1

            # print 'Selecting words of frequency <=', freq
            freq_stop = np.arange(cfs.size)[(cfs <= freq)]
            stop = set(freq_stop)
            for word in stop:
                stoplist.append(self.words[word])
        else:
            stop = set()

        # filter stoplist
        # print len(stoplist), "filtering to",
        stoplist = [t for t in stoplist if binary_search(self.words, t) >= 0]
        # print len(stoplist)
        for t in stoplist:
            stop.add(self.words_int[t])

        if not stop:
            # print 'Stop list is empty.'
            return self

        # print 'sorting stopwords', datetime.now() 
        stoplist = sorted(stoplist)
        stop = sorted(stop)
    
        # print 'Removing stop words', datetime.now()
        f = np.vectorize(lambda x: binary_search(stop, x) < 0)

        # print 'Rebuilding context data', datetime.now()
        context_data = []
        for i in xrange(len(self.context_data)):
            # print 'Recomputing token breaks:', self.context_types[i]
            tokens = self.view_contexts(self.context_types[i])
            # print self.context_types[i], len(stoplist), len(stop), datetime.now()
            spans = [t[f(t)].size if t.size else 0 for t in tokens]
            tok = self.context_data[i].copy()
            tok['idx'] = np.cumsum(spans)
            context_data.append(tok)

        del self.context_data
        self.context_data = context_data

        # print 'Rebuilding corpus and updating stop words', datetime.now()
        self.corpus = self.corpus[f(self.corpus)]
        self.stopped_words.update(stoplist)

        # print 'adjusting words list', datetime.now()
        new_words = np.array([t for t in self.words if binary_search(stoplist,t) < 0])

        # print 'rebuilding word dictionary', datetime.now()
        new_words_int = dict((word,i) for i, word in enumerate(new_words))

        # print "remapping corpus", datetime.now()
        current_offset = 0
        for i, tok in enumerate(self.corpus):
            self.corpus[i] = new_words_int[self.words[tok]] 
        # print len(self.words), len(self.words_int), len(new_words), len(new_words_int)

        # print 'storing new word dicts', datetime.now()
        self.words = new_words
        self.words_int = new_words_int

        return self

    def apply_stoplist(self, stoplist=[], freq=0):
        """ 
        Takes a Corpus object and returns a copy of it with words in the
        stoplist removed and with words of frequency <= `freq` removed.
        
        :param stoplist: The list of words to be removed.
        :type stoplist: list

        :param freq: A threshold where words of frequency <= 'freq' are
            removed. Default is 0.
        :type freq: integer, optional
            
        :returns: Copy of corpus with words in the stoplist and words
            of frequnecy <= 'freq' removed.

        :See Also: :class:`Corpus`
        """
        print "Using apply_stoplist for some reason"
        stoplist = set(stoplist)
        if freq:
            #TODO: Use the TF model instead

            # print 'Computing collection frequencies'
            cfs = np.zeros_like(self.words, dtype=self.corpus.dtype)
    
            for word in self.corpus:
                cfs[word] += 1

            # print 'Selecting words of frequency <=', freq
            freq_stop = np.arange(cfs.size)[(cfs <= freq)]
            stop = set(freq_stop)
            for word in stop:
                stoplist.add(self.words[word])
        else:
            stop = set()

        # filter stoplist
        stoplist = [t for t in stoplist if t in self.words]
        for t in stoplist:
            stop.add(self.words_int[t])

        if not stop:
            # print 'Stop list is empty.'
            return self
    
        # print 'Removing stop words'
        f = np.vectorize(lambda x: x not in stop)
        corpus = self.corpus[f(self.corpus)]

        # print 'Rebuilding corpus'
        corpus = [self.words[i] for i in corpus]
        context_data = []
        for i in xrange(len(self.context_data)):
            # print 'Recomputing token breaks:', self.context_types[i]
            tokens = self.view_contexts(self.context_types[i])
            spans = [t[f(t)].size for t in tokens]
            tok = self.context_data[i].copy()
            tok['idx'] = np.cumsum(spans)
            context_data.append(tok)

        c = CorpusSent(corpus, self.sentences, context_data=context_data, 
            context_types=self.context_types)
        if self.stopped_words:
            c.stopped_words.update(self.stopped_words)
        c.stopped_words.update(stoplist)
        return c

    @staticmethod
    def load(file):
        """
        Loads data into a Corpus object that has been stored using
        `save`.
        
        :param file: Designates the file to read. If `file` is a string ending
            in `.gz`, the file is first gunzipped. See `numpy.load`
            for further details.
        :type file: string-like or file-like object

        :returns: c : A Corpus object storing the data found in `file`.

        :See Also: :class: Corpus, :meth: Corpus.load, :meth: numpy.load
        """
        print 'Loading corpus from', file
        arrays_in = np.load(file)

        c = CorpusSent([], [])
        c.corpus = arrays_in['corpus']
        c.words = arrays_in['words']
        c.sentences = arrays_in['sentences']
        c.context_types = arrays_in['context_types'].tolist()
        try:
            c.stopped_words = set(arrays_in['stopped_words'].tolist())
        except:
            c.stopped_words = set()

        c.context_data = list()
        for n in c.context_types:
            t = arrays_in['context_data_' + n]
            c.context_data.append(t)

        c.__set_words_int()

        return c

    def save(self, file):
        """
        Saves data from a CorpusSent object as an `npz` file.
        
        :param file: Designates the file to which to save data. See
            `numpy.savez` for further details.
        :type file: str-like or file-like object
            
        :returns: None

        :See Also: :class: Corpus, :meth: Corpus.save, :meth: numpy.savez
        """
	
	print 'Saving corpus as', file
        arrays_out = dict()
        arrays_out['corpus'] = self.corpus
        arrays_out['words'] = self.words
        arrays_out['sentences'] = self.sentences
        arrays_out['context_types'] = np.asarray(self.context_types)
        arrays_out['stopped_words'] = np.asarray(self.stopped_words)

        for i,t in enumerate(self.context_data):
            key = 'context_data_' + self.context_types[i]
            arrays_out[key] = t

        np.savez(file, **arrays_out)
        
    
    def sent_int(self, sent):
        """
        sent : list of strings
            List of sentence tokenization.
            The list could be a subset of existing sentence tokenization.
        """
        tok = self.view_contexts('sentence', as_strings=True)
        sent_li = []
        for i in xrange(len(tok)):
            sent_li.append(sent)
        keys = [i for i in xrange(len(tok)) 
                if set(sent_li[i]).issubset(tok[i].tolist())]
        
        n = len(keys)
        if n == 0:
            raise Exception('No token fits {0}.'.format(sent))
        elif n > 1:
            return keys
        return keys[0] 

def make_sent_view_fn(corp):
    return (lambda md: 
        np.array(['{0}, {1}'.format(id, sent) 
            for id, sent in zip(md['sentence_label'], corp.sentences)]))
    

def sim_sent_sent(ldaviewer, sent, print_len=10, min_sent_len=0):
    """
    ldaviewer : ldaviewer object
    sent : sentence index or sentence as a list of words

    Returns
    -------
    sim_sents : numpy array
        (sentence index, probability) as (i, value) pair.
    tokenized_sents : list of arrays
        List containing tokenized sentences as arrays.
    orig_sents : list of strings
        List containing original sentences as strings.
    """
    from vsm.viewer.ldagibbsviewer import LDAGibbsViewer

    corp = ldaviewer.corpus
    ind = sent
    if isinstance(sent, list) and isinstance(sent[0], str):
        ind = corp.sent_int(sent)

    sentence_toks = corp.view_contexts('sentence', as_strings=True)
    sim_sents = ldaviewer.sim_doc_doc(ind, label_fn=make_sent_view_fn(corp), print_len=print_len)
    lc = sim_sents['doc']
    lc = [s.split(', ') for s in lc]
    lc = [int(s[0]) for s in lc]

    # only returns print_len length
    tokenized_sents, orig_sents = [], []
    for i in lc:
        if len(sentence_toks[i]) > min_sent_len and len(tokenized_sents) < print_len:
            tokenized_sents.append(sentence_toks[i])
            orig_sents.append(corp.sentences[i])

    mask = np.array([len(sentence_toks[i]) > min_sent_len for i in lc])
    sim_sents = sim_sents[mask]

    return tokenized_sents, orig_sents, sim_sents


def sim_sent_sent_across(ldavFrom, ldavTo, beagleviewer, sent, print_len=10,
                         label_fn=vol_link_fn):
    """
    ldavFrom : ldaviewer object where the sentence is from.
    ldavTo : ldaviewer object to find similar sentences
    beagleviewer : beagleviewer object is used to find
        similar words for words that don't exist in a different corpus.
    sent : sentence index of the corpus that corresponds to ldavFrom,
        or sentence as a list of words

    Returns
    -------
    sim_sents : numpy array
        (sentence index, probability) as (i, value) pair.
    tokenized_sents : list of arrays
        List containing tokenized sentences as arrays.
    orig_sents : list of strings
        List containing original sentences as strings.
    """
    from vsm.viewer.ldagibbsviewer import LDAGibbsViewer
    from vsm.viewer.beagleviewer import BeagleViewer

    def first_in_corp(corp, wordlist):
        """
        Goes down the list to find a word that's in `corp`. 
        Assumes there is a word in the `wordlist` that's in `corp`.
        """
        for w in wordlist:
            if w in corp.words:
                return w

    corp = ldavFrom.corpus # to get sent ind
    ind = sent
    word_list = []
    if isinstance(sent, list) and isinstance(sent[0], str):
        ind = corp.sent_int(sent)
        word_list = sent
    elif isinstance(sent, list):
        word_list = set()
        for i in sent:
            li = set(ldavFrom.corpus.view_contexts('sentence', 
                    as_strings=True)[i])
            word_list.update(li)       

    else: # if sent is an int index
        word_list = ldavFrom.corpus.view_contexts('sentence',
                    as_strings=True)[ind].tolist()

    word_list = list(word_list)
    # Before trying ldavTo.sim_word_word, make sure all words
    # in the list exist in ldavTo.corpus.
    wl = []
    for w in word_list:
        if w not in ldavTo.corpus.words:
            words = beagleviewer.sim_word_word(w)['word']
            replacement = first_in_corp(ldavTo.corpus, words)
            wl.append(replacement)
            print 'BEAGLE composite model replaced {0} by {1}'.format(w, 
                                                        replacement)
        else:
            wl.append(w)
    
    # from ldavFrom:sent -> ldavTo:topics -> ldavTo:sent(doc)
    tops = ldavTo.sim_word_top(wl).first_cols[:(ldavTo.model.K/6)]
    tops = [int(t) for t in tops]
    print "Related topics: ", tops 
    # sim_sents = ldavTo.sim_top_doc(tops, print_len=print_len, 
    #                                as_strings=False)
    # lc = sim_sents['i'][:print_len]
    # tokenized_sents, orig_sents = [], []
    # for i in lc:
    #    tokenized_sents.append(ldavTo.corpus.view_contexts('sentence', as_strings=True)[i])
    #   orig_sents.append(ldavTo.corpus.sentences[i])
    sim_sents = ldavTo.sim_top_doc(tops, print_len=print_len,
                                    label_fn=label_fn)
    return sim_sents


def extend_sdd(args, v, print_len=10):
    """
    Extend table resulting from sim_doc_doc with 
    label_fn = vol_link_fn. Adds an ArgumentMap column.
    """
    from vsm.viewer.ldagibbsviewer import LDAGibbsViewer

    sdd = v.sim_doc_doc(args, label_fn=vol_link_fn, print_len=print_len)
    table_str = sdd._repr_html_()
    rows = table_str.split('</tr>') 

    rows[0] = re.sub("2", "3", rows[0]) + '</tr>'
    rows[1] += '<th style="text-align: center; background: #EFF2FB;">Argument\
                Map</th></tr>'
    
    for i in xrange(2,len(rows)-1):
        a = rows[i].split('</a>, ')
        arg = a[1].split(',')[0]

        arg_map = find_arg(arg) 
        rows[i] += '<td>{0}</td></tr>'.format(arg_map)

    return ''.join(rows)


def extend_across(vFrom, vTo, beagle_v, args, txtFrom, txtTo, print_len=10):
    """
    Extend table resulting from sim_sent_sent_across with
    the label_fn= vol_link_fn. Adds ArgumentMap and Novelty columns.
    """
    from vsm.extensions.htrc import add_link_

    across = sim_sent_sent_across(vFrom, vTo, beagle_v, args, print_len=print_len)
    table_str = across._repr_html_()
    rows = table_str.split('</tr>') 

    rows[0] = re.sub("2", "4", rows[0]) + '</tr>'
    rows[1] += '<th style="text-align: center; background: #EFF2FB;">\
                Argument Map</th><th style="text-align: center; background: \
                #EFF2FB;">Novelty</th></tr>'
    
    for i in xrange(2,len(rows)-1):
        a = rows[i].split('</a>, ')
        arg = a[1].split(',')[0]

        novelty = in_ed1(arg, txtTo, txtFrom)
        arg_map = find_arg(novelty)
       
        # add link to novelty when it's found in the corpusFrom.
        if not novelty == 'new':
            li = novelty.split(' ')
            idx = int(li[0])
            md = vFrom.corpus.view_metadata('sentence')[idx]
            link = add_link_(md['page_urls'], md['sentence_label'])
            li[0] = link
            novelty = ' '.join(li)

        rows[i] += '<td>{0}</td><td>{1}</td></tr>'.format(arg_map, novelty)

    return ''.join(rows)

  
def in_ed1(idx, difftxt, ed1txt):
    """
    Only for sim_sent_sent_across.
    Return ind from ed1txt if i has a equal match.
    Else return 'i'th entry in difftxt.
    
    """
    path = '/var/inphosemantics/data/20131214/Washburn/vsm-data/'
    
    with open(path + ed1txt, 'r') as f1:
        ed1 = f1.read()
        ed1 = ed1.split(',') 
        
        with open(path + difftxt, 'r') as f:
            txt = f.read() 
            entries = txt.split(',')
            
            for i in xrange(len(entries)): 
                if entries[i].startswith(str(idx) + ' '):
                    if '=' in entries[i]:
                        return ed1[i]
                    else:
                        prob = entries[i].split(' ')[1]
                        return ed1[i] + ' ' + prob
            # didn't find idx in the table.
            return 'new'

def find_arg(i):
    """
    Find the arg (e.g. '422')  if i is one of the analyzed args,
    otherwise return ''
    """
    import json

    path = '/var/inphosemantics/data/20131214/Washburn/vsm-data/'
    
    if i == 'new' or '(' in i:
        return ''

    i = int(i)
    with open(path + 'arg_indices.json', 'r') as jsonf:
        indices = json.load(jsonf)
        
        for k in indices:
            if i in indices[k]:
                return str(k)
        return ''


def file_tokenize(text):
    """
    `file_tokenize` is a helper function for :meth:`file_corpus`.
    
    Takes a string that is content in a file and returns words
    and corpus data.

    :param text: Content in a plain text file.
    :type text: string

    :returns: words : List of words.
        Words in the `text` tokenized by :meth:`vsm.corpus.util.word_tokenize`.
        corpus_data : Dictionary with context type as keys and
        corresponding tokenizations as values. The tokenizations
        are np.arrays.
    """
    words, par_tokens, sent_tokens, sent_orig = [], [], [], []
    sent_break, par_n, sent_n = 0, 0, 0

    pars = paragraph_tokenize(text)

    for par in pars:
        par = par.replace('\n',' ')
        sents = sentence_tokenize(par)

        for sent in sents:
            w = word_tokenize(sent)
            words.extend(w)
            sent_break += len(w)
            sent_tokens.append((sent_break, par_n, sent_n))
            sent_orig.append(sent)
            sent_n += 1

        par_tokens.append((sent_break, par_n))
        par_n += 1

    idx_dt = ('idx', np.int32)
    sent_label_dt = ('sentence_label', np.array(sent_n, np.str_).dtype)
    par_label_dt = ('paragraph_label', np.array(par_n, np.str_).dtype)

    corpus_data = dict()
    dtype = [idx_dt, par_label_dt]
    corpus_data['paragraph'] = np.array(par_tokens, dtype=dtype)
    dtype = [idx_dt, par_label_dt, sent_label_dt]
    corpus_data['sentence'] = np.array(sent_tokens, dtype=dtype)

    return words, corpus_data, sent_orig


def toy_corpus(filename, is_filename=True, nltk_stop=True, stop_freq=1,
    add_stop=None, decode=False, autolabel=False):
    if is_filename:
        return file_corpus(filename, nltk_stop=nltk_stop, stop_freq=stop_freq,
        add_stop=add_stop)

def file_corpus(filename, nltk_stop=True, stop_freq=1, add_stop=None):
    """
    `file_corpus` is a convenience function for generating Corpus
    objects from a a plain text corpus contained in a single string
    `file_corpus` will strip punctuation and arabic numerals outside
    the range 1-29. All letters are made lowercase.

    :param filename: File name of the plain text file.
    :type plain_dir: string-like
    
    :param nltk_stop: If `True` then the corpus object is masked 
        using the NLTK English stop words. Default is `False`.
    :type nltk_stop: boolean, optional
    
    :param stop_freq: The upper bound for a word to be masked on 
        the basis of its collection frequency. Default is 1.
    :type stop_freq: int, optional
    
    :param add_stop: A list of stop words. Default is `None`.
    :type add_stop: array-like, optional

    :returns: c : a Corpus object
        Contains the tokenized corpus built from the input plain-text
        corpus. Document tokens are named `documents`.
    
    :See Also: :class:`vsm.corpus.Corpus`, 
        :meth:`file_tokenize`, 
        :meth:`vsm.corpus.util.apply_stoplist`
    """
    with open(filename, mode='r') as f:
        text = f.read().replace('\r\n','\n')

    words, tok, sent = file_tokenize(text)
    names, data = zip(*tok.items())
    
    c = CorpusSent(words, sent, context_data=data, context_types=names,
                    remove_empty=False)
    in_place_stoplist(c, nltk_stop=nltk_stop,
                       freq=stop_freq, add_stop=add_stop)

    return c



def dir_tokenize(chunks, labels, chunk_name='article', paragraphs=True):
    """
    """
    words, chk_tokens, sent_tokens, sent_orig = [], [], [], []
    sent_break, chk_n, sent_n = 0, 0, 0

    if paragraphs:
        par_tokens = []
        par_n = 0
        
        for chk, label in zip(chunks, labels):
            print 'Tokenizing', label
            pars = paragraph_tokenize(chk)

            for par in pars:
                sents = sentence_tokenize(par)

                for sent in sents:
                    w = word_tokenize(sent)
                    words.extend(w)
                    sent_break += len(w)
                    sent_tokens.append((sent_break, label, par_n, sent_n))
                    sent_orig.append(sent)
                    sent_n += 1

                par_tokens.append((sent_break, label, par_n))
                par_n += 1

            chk_tokens.append((sent_break, label))
            chk_n += 1
    else:
        for chk, label in zip(chunks, labels):
            print 'Tokenizing', label
            sents = sentence_tokenize(chk)

            for sent in sents:
                w = word_tokenize(sent)
                words.extend(w)
                sent_break += len(w)
                sent_tokens.append((sent_break, label, sent_n))
                sent_orig.append(sent)
                sent_n += 1

            chk_tokens.append((sent_break, label))
            chk_n += 1

    idx_dt = ('idx', np.int32)
    label_dt = (chunk_name + '_label', np.array(labels).dtype)
    sent_label_dt = ('sentence_label', np.array(sent_n, np.str_).dtype)
    corpus_data = dict()
    dtype = [idx_dt, label_dt]
    corpus_data[chunk_name] = np.array(chk_tokens, dtype=dtype)

    if paragraphs:
        par_label_dt = ('paragraph_label', np.array(par_n, np.str_).dtype)
        dtype = [idx_dt, label_dt, par_label_dt]
        corpus_data['paragraph'] = np.array(par_tokens, dtype=dtype)
        dtype = [idx_dt, label_dt, par_label_dt, sent_label_dt]
        corpus_data['sentence'] = np.array(sent_tokens, dtype=dtype)
    else:
        dtype = [idx_dt, label_dt, sent_label_dt]
        corpus_data['sentence'] = np.array(sent_tokens, dtype=dtype)

    return words, corpus_data, sent_orig



def dir_corpus(plain_dir, chunk_name='article', paragraphs=True, word_len=2,
               nltk_stop=True, stop_freq=1, add_stop=None, corpus_sent=True,
               ignore=['.log', '.pickle', '.xml'], decode=False):
    """
    `dir_corpus` is a convenience function for generating Corpus
    objects from a directory of plain text files.

    `dir_corpus` will retain file-level tokenization and perform
    sentence and word tokenizations. Optionally, it will provide
    paragraph-level tokenizations.

    It will also strip punctuation and arabic numerals outside the
    range 1-29. All letters are made lowercase.

    :param plain_dir: String containing directory containing a 
        plain-text corpus.
    :type plain_dir: string-like
    
    :param chunk_name: The name of the tokenization corresponding 
        to individual files. For example, if the files are pages 
        of a book, one might set `chunk_name` to `pages`. Default 
        is `articles`.
    :type chunk_name: string-like, optional
    
    :param paragraphs: If `True`, a paragraph-level tokenization 
        is included. Defaults to `True`.
    :type paragraphs: boolean, optional
    
    :param word_len: Filters words whose lengths are <= word_len.
        Default is 2.
    :type word_len: int, optional

    :param nltk_stop: If `True` then the corpus object is masked 
        using the NLTK English stop words. Default is `False`.
    :type nltk_stop: boolean, optional
    
    :param stop_freq: The upper bound for a word to be masked on 
        the basis of its collection frequency. Default is 1.
    :type stop_freq: int, optional

    :param corpus_sent: If `True` a CorpusSent object is returned.
        Otherwise Corpus object is returned. Default is `True`. 
    :type corpus_sent: boolean, optional

    :param add_stop: A list of stop words. Default is `None`.
    :type add_stop: array-like, optional

    :param ignore: The list containing suffixes of files to be filtered.
        The suffix strings are normally file types. Default is ['.json',
        '.log', '.pickle'].
    :type ignore: list of strings, optional

    :returns: c : Corpus or CorpusSent
        Contains the tokenized corpus built from the input plain-text
        corpus. Document tokens are named `documents`.
    
    :See Also: :class: Corpus, :class: CorpusSent, :meth: dir_tokenize,
        :meth: apply_stoplist
    """
    chunks = []
    filenames = os.listdir(plain_dir)
    filenames = filter_by_suffix(filenames, ignore)
    filenames.sort()

    for filename in filenames:
        filename = os.path.join(plain_dir, filename)
        with open(filename, mode='r') as f:
            chunks.append(f.read())

    words, tok, sent = dir_tokenize(chunks, filenames, chunk_name=chunk_name,
                              paragraphs=paragraphs)
    names, data = zip(*tok.items())
    
    if corpus_sent:
        c = CorpusSent(words, sent, context_data=data, context_types=names,
			remove_empty=False)
    else:
        c = Corpus(words, context_data=data, context_types=names)
    
    in_place_stoplist(c, nltk_stop=nltk_stop, add_stop=add_stop, freq=stop_freq)

    return c



import numpy as np
from sklearn.cluster import KMeans
from sklearn import datasets
from random import randrange
from scipy.sparse import csgraph as cs


# generate sample data
n_samples = 30
noisy_circles = datasets.make_circles(n_samples=n_samples, factor=.5,
                                      noise=.05)
blobs =  datasets.make_blobs(n_samples=n_samples, random_state=8)

# S = noisy_circles[0]
S = blobs[0]

def multi_k(samples, n=30, distr=(10,30), cutoff=None, n_cls=None):
    """
    Oveall procedure run. Returns cutplot and category_func.
    """
    
    import matplotlib.pyplot as plt

    mat, cutplot = connection_matrix(samples=samples, n=n, distr=distr)

    if cutoff == None:
        cutoff = find_cutoff(cutplot, n_cls=n_cls)
    print "Weight cutoff is set to ", cutoff
    category_func = category_mat(samples, mat, cutplot, cutoff=cutoff)

    x, y = zip(*cutplot)
    plt.plot(x,y)

    return plt, cutplot, category_func


def connection_matrix(samples, n=10, distr=(10,30)):
    """
    samples : set S of r points in Rn.
    n : number n of clusterings to perform.
    distr : numbers of clusters.
        <- this should be a distribution over 0,...,n. 
        Let us use a uniform integer for a moment. 
    cutoff : weight cutoff, 0 <= cutoff <= 1.
    
    could make n,distr,cutoff optional by setting default values.
    
    :returns: mat : connection matrix 
             cutplot :  A cut plot
                f : [0,1] -> Z
    """
    r = len(samples)
    mat = np.zeros((r, r))
    
    for n_ in xrange(n):
        # select integer d from distr[0] to distr[1] 
        d = randrange(distr[0],distr[1]+1)
        km = KMeans(n_clusters=d, init='k-means++',
                    max_iter=100, n_init=1,verbose=False)
        km.fit(samples)
        labels = km.labels_

        for i in range(r):
            for j in range(i,r):
                if labels[i] == labels[j]:
                    mat[i][j] += 1
        
    mat = mat + mat.T
    mat /= n

    cutplot = np.zeros((n+1 ,2), dtype='f2<')
    for l in xrange(n+1):
        # Construct graph for which mat[i][j] > l/n
        graph = mat > 1.0 * l /n
        
        n_comp, labels = cs.cs_graph_components(graph)
        cutplot[l][0] = l * 1.0 /n
        cutplot[l][1] = n_comp
    return mat, cutplot


def find_cutoff(cutplot, n_cls=None):
    """
    Finds the weight cutoff based on the longest run in cutplot.
    If n_cls is provided, finds the cutoff point where n_cls
    clusters are formed."""
    from itertools import groupby

    if n_cls != None:
        for c in cutplot:
            if c[1] == n_cls:
                return c[0]

    group = groupby(cutplot[:,1])
    val = max(group, key=lambda k: len(list(k[1])))[0]
    
    for c in cutplot:
        if c[1] == val:
            return c[0] + 0.01


def category_mat(samples, mat, cutplot, cutoff=None):
    """
    Predicts the category for each data point
    """ 
    if cutoff == None:
        cutoff = find_cutoff(cutplot)
    # Build a new graph on samples with edges mat[i][j] > cutoff
    newG = mat > cutoff
    n_comp, labels = cs.cs_graph_components(newG)
    
    return labels
   



import numpy as np
from scipy import sparse
from scipy.sparse import linalg as linalgs


__all__ = [ 'Lsa' ]


class Lsa(object):
    """
    """
    
    def __init__(self, corpus=None, context_type=None, td_matrix=None):
        """
        Initialize Lsa.

        :param corpus: A Corpus object containing the training data.
        :type corpus: Corpus, optional

        :param context_type: Name of tokenization whose tokens will be
            treated as documents. Default is `None`.
        :type context_type: string, optional

        :param td_matrix: Term-Document matrix. Default is `None`.
        :type td_matrix: np.array, optional
        """

        self.word_matrix = None
        self.doc_matrix = None
        self.eigenvalues = None
        self.context_type = context_type
        if corpus is not None:
            self.corpus = corpus.corpus
        else:
            self.corpus = []

        if td_matrix is None:
            self.td_matrix = np.array([])
        else:
            td_matrix = sparse.coo_matrix(td_matrix)
            
            # Removing infinite values for SVD
            finite_mask = np.isfinite(td_matrix.data)
            coo_in = (td_matrix.data[finite_mask],
                      (td_matrix.row[finite_mask], 
                       td_matrix.col[finite_mask]))

            td_matrix = sparse.coo_matrix(coo_in, shape=td_matrix.shape, 
                                          dtype=np.float64)
            self.td_matrix = td_matrix.tocsr()


    def train(self, k_factors=300):
        """
        Trains the model.

        :param k_factors: Default is 300.
        :type k_factors: int, optional
        """

        u,s,v = np.array([]), np.array([]), np.array([])

        if self.td_matrix.size > 0:
            s = min(self.td_matrix.shape)
            if s < k_factors:
                k_factors = s - 1

            # print 'Performing sparse SVD'
            u, s, v = linalgs.svds(self.td_matrix, k=k_factors)

        indices = s.argsort()[::-1]
        self.word_matrix = u[:, indices]
        self.eigenvalues = s[indices]
        self.doc_matrix = v[indices, :]


    def save(self, f):
        """
        Saves model data as a numpy archive file with extension `npz`.
        The keys for the component matrices are `word_matrix`,
        `eigenvalues` and `doc_matrix`.
        
        :param f: Designates the file to which to save data. See
            `numpy.savez` for further details.
        :type f: str-like or file-like object
            
        :See Also: :meth:`numpy.savez`
        """
        arrays_out = dict()
        arrays_out['word_matrix'] = self.word_matrix
        arrays_out['eigenvalues'] = self.eigenvalues
        arrays_out['doc_matrix'] = self.doc_matrix
        arrays_out['context_type'] = self.context_type
        np.savez(f, **arrays_out)


    @staticmethod
    def load(f):
        """
        Loads LSA model data from a numpy archive file with extension
        `npz`. The expected keys for the component matrices are
        `word_matrix`, `eigenvalues` and `doc_matrix`.
        
        :param f: Designates the file from which to load data. See
            `numpy.load` for further details.
        :type f: str-like or file-like object
            
        :returns: a saved Lsa model.

        :See Also: :meth:`numpy.load`
        """
        arrays_in = np.load(f)
        m = Lsa(context_type=arrays_in['context_type'])
        m.word_matrix=arrays_in['word_matrix']
        m.eigenvalues=arrays_in['eigenvalues']
        m.doc_matrix=arrays_in['doc_matrix']
        return m

    @staticmethod
    def from_tf(tf_model):
        """
        Takes a `Tf` model object and generates a `TfIdf` model.
        """
        model = Lsa(td_matrix=tf_model.matrix)
        model.corpus = tf_model.corpus
        model.context_type = tf_model.context_type
        return model
    
    @staticmethod
    def from_tfidf(tfidf_model):
        """
        Takes a `Tf` model object and generates a `TfIdf` model.
        """
        model = Lsa(td_matrix=tfidf_model.matrix)
        model.corpus = tfidf_model.corpus
        model.context_type = tfidf_model.context_type
        return model



import numpy as np
from sklearn.cluster import KMeans, AffinityPropagation, SpectralClustering
from sklearn.manifold import Isomap, MDS
from plotting import plot_clusters


__all__ = [ 'Manifold' ]


class Manifold(object):
    def __init__(self, dismat, labels=None, cls=[], pos=[]):
        self.dismat = np.asarray(dismat)
        self.labels = labels
        self._cls = cls     # Clusters info
        self.pos = pos


    def __str__(self):
        return self.dismat.__str__()


    @property
    def cls(self):
        """
        views clusters as lists
        """
        return [[self.labels[i] for i,lab in enumerate(self._cls) if lab == x]
                for x in set(self._cls)]


#
# Clustering methods
#
    def KMeans(self, n_clusters=10, init='k-means++', max_iter=100,
               n_init=1, verbose=1, show=True):
        """
        Clusters the objects in `dismat` using k-means algorithm. This requires 
        `pos` be  precomputed by `mds` or `isomap`. For parameters of the 
        algorithms see: 
        http://scikit-learn.org/stable/modules/generated/sklearn.cluster.KMeans.
        html#sklearn.cluster.KMeans

        :param n_clusters: Number of clusters used as the parameter for K-means.
        :type n_clusters: int, optional

        :param show: Shows the resulting clusters if true.
        :type n_clusters: boolean, optional
        """

        if len(self.pos)==0:
            raise Exception('K-Means requires low dimentional coordinates. Try mds() or isomap() first.')

        model = KMeans(n_clusters=n_clusters, init=init, max_iter=max_iter, 
                       n_init=n_init,verbose=verbose).fit(self.pos)
        self._cls = model.labels_

        if show:
            return self.cls
        


    def AffinityPropagation(self, show=True):
        """
        Clusters objects in `dismat` using affinity propagation algorithm.

        :param show: Shows the resulting clusters if true.
        :type n_clusters: boolean, optional
        """

        model = AffinityPropagation(affinity='precomputed').fit(self.dismat)
        self._cls = model.labels_
        
        if show:
            return self.cls



    def SpectralClustering(self, n_clusters=10, show=True):
        """
        Clusters objects in `dismat` using spectral clustering. 

        :param n_clusters: Number of clusters used as the parameter for K-means.
        :type n_clusters: int, optional

        :param show: Shows the resulting clusters if true.
        :type n_clusters: boolean, optional
        """

        model = SpectralClustering(n_clusters=n_clusters, 
                                   affinity='precomputed').fit(self.dismat)
        self._cls = model.labels_

        if show:
            return self.cls



#
# Manifold learning methods
#

    def mds(self, n_components=2, dissimilarity='precomputed', show=False): 
        """
        Calculates lower dimention coordinates using the mds algorithm.
        This requires sklearn ver 0.14 due to the dissimilarity argument.

        :param n_components: dimentionality of the reduced space.
        :type n_components: int, optional

        :param show: Shows the calculated coordinates if true.
        :type show: boolean, optional
        """
        model = MDS(n_components=n_components, dissimilarity=dissimilarity, max_iter=100)
        self.pos = model.fit_transform(self.dismat)

        if show:
            return self.pos



    def isomap(self, n_components=2, n_neighbors=3, show=False):
        """
        Calculates lower dimention coordinates using the isomap algorithm.

        :param n_components: dimentionality of the reduced space
        :type n_components: int, optional

        :param n_neighbors: Used by isomap to determine the number of neighbors
            for each point. Large neighbor size tends to produce a denser map.
        :type n_neighbors: int, optional

        :param show: Shows the calculated coordinates if true.
        :type show: boolean, optional
        """

        model = Isomap(n_components=n_components, n_neighbors=n_neighbors)
        self.pos  = model.fit(self.dismat).embedding_

        if show:
            return self.pos



    def plot(self, xy = (0,1)):
        """
        Outputs 2d embeded plot based on `pos`

        :param xy: specifies the dimsntions of pos to be plotted.
        :type xy: tuple, optional

        """
        return plot_clusters(self.pos[:,[xy[0],xy[1]]], self.labels, clusters=self._cls)



"""
[General documentation about the :mod:`viewer` submodule]
"""


import beagleviewer
from beagleviewer import *
import ldacgsviewer
from ldacgsviewer import *
import lsaviewer
from lsaviewer import *
import tfviewer
from tfviewer import *
import tfidfviewer
from tfidfviewer import *


__all__ = beagleviewer.__all__[:]
__all__ += ldacgsviewer.__all__
__all__ += lsaviewer.__all__
__all__ += tfviewer.__all__
__all__ += tfidfviewer.__all__
 





"""
[Documentation for structarr]
"""


import numpy as np


__all__ = ['arr_add_field', 'enum_array', 'enum_matrix', 
           'enum_sort', 'map_strarr', 'zip_arr']


def arr_add_field(arr, new_field, vals):
    """
    Adds a new field to a structured array.
    This is a handy function for adding new metadata to 
    the original metadata array in Corpus object.
    
    :param arr: A structured array.
    :type arr: array
    
    :param new_field: The dtype name for the values.
    :type new_field: string
    
    :param vals: A list of values. `vals` must have the same length as `arr`.
    :type vals: list

    :returns: new_arr : New array with the added values.

    :See Also: :meth:`view_metadata`

    **Examples**

    >>> arr = np.array([('a', 1), ('b', 2), ('c', 3)], 
            dtype=[('field', '|S4'), ('i', '<i4')])
    >>> vals = [-1, -2, -3]
    >>> arr_add_field(arr, 'neg_i', vals)
    array([('a', 1, -1), ('b', 2, -2), ('c', 3, -3)], 
          dtype=[('field', '|S4'), ('i', '<i4'), ('neg_i', '<i8')])
    """
    # Constructing new dtype
    new_dtype = np.array(vals).dtype
    dt = [(n, arr.dtype[n]) for n in arr.dtype.names]
    dt.append((new_field, new_dtype))

    # Building new structured array
    new_arr = np.empty_like(arr, dtype=dt)
    for n in new_arr.dtype.names:
        if n == new_field:
            new_arr[n][:] = vals[:]
        else:
            new_arr[n][:] = arr[n][:]

    return new_arr
    

def enum_array(arr, indices=[], field_name='i'):
    """
    Takes an array and returns a structured array with indices
    and values as dtype names.

    :param arr: 1-dimensional array
    :type arr: array
    
    :param indices: List of indices. If `indices` is empty, then `indices` 
        is set to a range of indices for the length of `arr`. 
        Default is an empty list.
    :type indices: list, optional
    
    :param field_name: Name for indices in the structured array dtype. 
        Default is 'i'.
    :type field_name: string, optional 

    :returns: A structured array with indices and value fields.

    :See Also: :meth:`zip_arr`

    **Examples**

    >>> arr = np.array([7,3,1,8,2])
    >>> enum_array(arr)
    array([(0, 7), (1, 3), (2, 1), (3, 8), (4, 2)], 
          dtype=[('i', '<i8'), ('value', '<i8')])
    """
    if len(indices) == 0:
        indices = np.arange(arr.size)
    else:
        indices = np.array(indices)
    return zip_arr(indices, arr, field_names=[field_name, 'value'])


def enum_matrix(arr, axis=0, indices=[], field_name='i'):
    """
    Takes a 1-dimensional or 2-dimensional array and returns a sorted
    structured array with indices.

    :param arr: 1-dimensional or 2-dimensional numpy array.
    :type arr: array
    
    :param axis: Array axis 0 or 1. Default is 0.
    :type axis: int, optional
    
    :param indices: List of indices. If 'indices' is empty, then `indices` 
        is set to a range of indices for the length of `arr`. 
        Default is an empty list.
    :type indices: list, optional
    
    :param field_name: Name for indices in the structured array dtype. 
        Default is 'i'.
    :type field_name: string, optional

    :returns: mt : A sorted structured array with indices.

    :See Also: :meth:`zip_arr`

    **Examples**

    >>> arr = np.array([[6,3,7],[2,0,4]])
    >>> enum_matrix(arr)
    array([[(2, 7), (0, 6), (1, 3)],
           [(2, 4), (0, 2), (1, 0)]], 
           dtype=[('i', '<i8'), ('value', '<i8')])
    """
    #if len(indices) == 0:
    #    indices = np.arange(arr.shape[1])
    if len(indices) == 0 and len(arr.shape) > 1:
        indices = np.arange(arr.shape[1])
    
    if type(indices) == list:
        indices = np.array(indices)
    ind = np.array([indices.copy() for i in xrange(arr.shape[0])])
    dt = [(field_name, indices.dtype), ('value', arr.dtype)]
    mt = zip_arr(ind, arr, field_names=[field_name, 'value'])

    if len(arr.shape) > 1:
        if axis:
            for i in xrange(arr.shape[axis]):
                idx = np.argsort(mt['value'][:,i])
                mt[field_name][:,i] = ind[:,i][idx]
                mt['value'][:,i] = arr[:,i][idx]
                mt[:,i] = mt[:,i][::-1]	
        else:
            for i in xrange(arr.shape[axis]):
                idx = np.argsort(mt['value'][i])
                mt[field_name][i] = ind[i][idx]
                mt['value'][i] = arr[i][idx]
                mt[i,:] = mt[i,:][::-1]
    else:
        idx = np.argsort(arr)
        mt[field_name][:] = idx[:]
        mt['value'][:] = arr[idx]

    return mt


def enum_sort(arr, indices=[], field_name='i', filter_nan=False):
    """
    Takes a 1-dimensional array and returns a sorted array with matching
    indices from the original array.

    :param arr: A structured 1-dimensional array.
    :type arr: array
    
    :param indices: List of indices. If `indices` is empty, then `indices`
        is set to a range of indices for the length of `arr`. 
        Default is an empty list.
    :type indices: list, optional
    
    :param field_name: Name for indices in the structured array dtype. 
        Default is 'i'.
    :type field_name: string, optional
    
    :param filter_nan: If `True`, Not a Number values are filtered. 
        Default is `False`.
    :type filter_nan: boolean, optional

    :returns: A sorted structured array.

    **Examples**

    >>> arr = np.array([7,3,1,8,2])
    >>> enum_sort(arr)
    array([(3, 8), (0, 7), (1, 3), (4, 2), (2, 1)], 
          dtype=[('i', '<i8'), ('value', '<i8')])
    """
    idx = np.argsort(arr)
    if len(indices) == 0:
        indices = np.arange(arr.shape[0])
    else:
        indices = np.array(indices)
	
    dt = [(field_name, indices.dtype), ('value', arr.dtype)]

    new_arr = np.empty(shape=arr.shape, dtype=dt)
    new_arr[field_name] = indices[idx]
    new_arr['value'] = arr[idx]

    if filter_nan:
        new_arr = new_arr[np.isfinite(new_arr['value'])]
        
    return new_arr[::-1]
 

def map_strarr(arr, m, k, new_k=None):
    """
    Takes a structured array `arr`, a field name `k` and an Indexable
    `m` and returns a copy of `arr` with its field `k` values mapped
    according to `m`. If `new_name` is given, the field name `k` is
    replaced with `new_name`.

    :param arr: A structured array.
    :type arr: array

    :type m: iterable
    :param m: An indexable array or list to retrieve the values from.
        The iterable contains values to replace the original `k` values.
    :type k: string
    :param k: Field name of `arr`. arr[k] are the values to be replaced. 
        arr[k] should be an array of integers.
    :type new_k: string, optional
    :param new_k: Field name for the new values. If not provided, field name
        is then set to `k`, the original field name. Default is `None`.

    :returns: new_ arr : A new array with `k` values replaced by values in `m`.

    **Examples**

    >>> arr = np.array([(0, 1.), (1, 2.)], dtype=[('i', 'i4'), ('v', 'f4')])
    >>> m = ['foo', 'bar']
    >>> map_strarr(arr, m, 'i', 'string')
    array([('foo', 1.0), ('bar', 2.0)], 
          dtype=[('string', '|S3'), ('v', '<f4')])
    """
    # Constructing new dtype
    if not new_k:
        new_k = k
    k_vals = np.array([m[i] for i in arr[k]])
    dt = [(n, arr.dtype[n]) for n in arr.dtype.names]
    i = arr.dtype.names.index(k)
    dt[i] = (new_k, k_vals.dtype)

    # Building new structured array
    new_arr = np.empty_like(arr, dtype=dt)
    for n in new_arr.dtype.names:
        if n == new_k:
            new_arr[new_k][:] = k_vals[:]
        else:
            new_arr[n][:] = arr[n][:]

    return new_arr


def zip_arr(arr_1, arr_2, field_names=['arr_1','arr_2']):
    """
    Takes two arrays with same shape and returns a zipped structured array.

    :param arr_1: 1-dimensional array.        
    :type arr_1: array
    
    :param arr_2: 1-dimensional array.
    :type arr_2: array
    
    :param field_names: List of numpy dtype names.
    :type field_names: list, optional

    :returns: new_arr : Zipped array of `arr_1` and `arr_2`.

    **Examples**

    >>> a1 = np.array([[2,4], [6,8]])
    >>> a2 = np.array([[1,3],[5,7]])
    >>> zip_arr(a1, a2, field_names=['even', 'odd'])
    array([[(2, 1), (4, 3)],
           [(6, 5), (8, 7)]], 
           dtype=[('even', '<i8'), ('odd', '<i8')])
    """
    if field_names:
        dt = [(field_names[0], arr_1.dtype), (field_names[1], arr_2.dtype)]

    new_arr = np.empty_like(arr_1, dtype=dt)

    new_arr[new_arr.dtype.names[0]][:] = arr_1[:]
    new_arr[new_arr.dtype.names[1]][:] = arr_2[:]
    
    return new_arr



"""
[General Documentation about :mod:`model` classes]
"""

import beaglecomposite
from beaglecomposite import *
import beaglecontext
from beaglecontext import *
import beagleenvironment
from beagleenvironment import *
import beagleorder
from beagleorder import *
import lda
from lda import *
import ldacgsseq
from ldacgsseq import *
import ldacgsmulti
from ldacgsmulti import *
import lsa
from lsa import *
import tf
from tf import *
import tfidf
from tfidf import *


__all__ = beaglecomposite.__all__[:]
__all__ += beaglecontext.__all__
__all__ += beagleenvironment.__all__
__all__ += beagleorder.__all__
__all__ += lda.__all__
__all__ += ldacgsseq.__all__
__all__ += ldacgsmulti.__all__
__all__ += lsa.__all__
__all__ += tf.__all__
__all__ += tfidf.__all__



import numpy as np
from itertools import product
from ldacgsseq import LdaCgsSeq


__all__ = [ 'LdaExact' ]



def uniquify(l):
    """Takes a list `l` and returns a list of the unique elements in `l`
    in the order in which they appeared.

    """
    mem = set([])
    out = []
    for e in l:
        if e not in mem:
            mem.add(e)
            out.append(e)
    return out


def productoid(A, n):

    prod = product(A, repeat=n)
    d = dict((i, A[:i]) for i in xrange(1, len(A)+1))

    for t in prod:
        elems = uniquify(t)
        if elems == d[len(elems)]:
            yield t


class LdaExact(LdaCgsSeq):


    @property
    def arg_maxima(self):
        if hasattr(self, '_arg_maxima'):
            return self._arg_maxima
        return []


    @arg_maxima.setter
    def arg_maxima(self, l):
        self._arg_maxima = l


    def _Z_values(self):
        
        A = range(self.K)
        p = productoid(A, len(self.corpus))
        for t in p:
            yield np.array(t, dtype=np.int)


    def _init_model(self, Z):
        m = LdaCgsSeq(context_type=self.context_type,
                      K=self.K, V=self.V, alpha=self.alpha, beta=self.beta)
        m.corpus = self.corpus
        m.V = self.V
        m.indices = self.indices
        m.Z = Z
        m._compute_top_doc()
        m._compute_word_top()
        m.inv_top_sums = 1. / self.word_top.sum(0)
        m.iteration = 1
        m.log_probs = [(1, m._compute_log_prob())]
        return m
        

    def _log_probs(self):

        Z = self._Z_values()

        for next_Z in Z:
            m = self._init_model(next_Z)
            yield (next_Z, m.log_probs[0][1])


    def arg_max(self, verbose=1):
        
        max_log_prob = -np.inf
        maxima = []

        log_probs = self._log_probs()
        for (Z, log_prob) in log_probs:
            if log_prob == max_log_prob:
                maxima.append((Z, log_prob))
            elif log_prob > max_log_prob:
                max_log_prob = log_prob
                maxima = [(Z, log_prob)]

        self.arg_maxima = maxima
        self.Z = maxima[0][0]
        self._compute_top_doc()
        self._compute_word_top()
        self.inv_top_sums = 1. / self.word_top.sum(0)
        self.iteration = 1
        self.log_probs = [(1, max_log_prob)]

        if verbose > 0:
            print 'Number of maxima:', len(self.arg_maxima)


    def all_estimates(self):
        
        for (Z, log_prob) in self.arg_maxima:
            yield self._init_model(Z)




import os
import shutil
import tempfile
import multiprocessing as mp
import cPickle as cpickle

import numpy as np
from numpy import dual

from vsm.spatial import rand_pt_unit_sphere
from base import BaseModel


__all__ = [ 'BeagleOrderSeq', 'BeagleOrderMulti' ]



def two_rand_perm(n, seed=None):
    """
    """
    np.random.seed(seed)
    
    perm1 = np.random.permutation(n)

    while True:
        perm2 = np.random.permutation(n)

        if not (perm2 == perm1).all():
            break
        
    return perm1, perm2


def mk_b_conv(n, rand_perm=None):
    """
    """
    if rand_perm is None:
        rand_perm = two_rand_perm(n)
    
    def b_conv(v1, v2):
        
        w1 = dual.fft(v1[rand_perm[0]])
        w2 = dual.fft(v2[rand_perm[1]])

        return np.real_if_close(dual.ifft(w1 * w2))

    return b_conv


def ngram_slices(i, n, l):
    """
    Given index i, n-gram width n and array length l, returns slices
    for all n-grams containing an ith element
    """
    out = []

    a = i - n + 1
    if a < 0:
        a = 0

    b = i + 1
    if b + n > l:
        b = l - n + 1

    d = b - a

    for k in xrange(d):
        start = a + k
        stop = start + n
        out.append(slice(start, stop))

    return out


def reduce_ngrams(fn, a, n, i, flat=True):
    """
    Given array a, reduce with fn all n-grams with width n and less and
    that contain the element at i.

    Memoizes.
    """
    m = n if n < a.shape[0] else a.shape[0]

    out = { 1: { i: a[i] } }
    
    for j in xrange(2, m + 1):

        slices = ngram_slices(i, j, a.shape[0])
        init = slices[0]
        out[j] = { init.start: reduce(fn, a[init]) }

        for s in slices[1:]:
            prev = out[j - 1][s.start]
            out[j][s.start] = fn(prev, a[s.stop - 1])

    # Delete the 1-gram
    del out[1]

    if flat:
        out = [v for d in out.values() for v in d.values()]
    
    return out


class BeagleOrderSeq(BaseModel):
    """
    `BeagleOrderSeq` stores word order information in the context.
    """

    def __init__(self, corpus, env_matrix, context_type='sentence',
                 psi=None, rand_perm=None, lmda =7):
        """ 
        Initialize BeagleOrderSeq.
        
        :param corpus: Soure of observed data.
        :type corpus: Corpus

        :param env_matrix: BEAGLE environment matrix.
        :type env_matrix: 2-D array

        :param context_type: Name of tokenization stored in `corpus` whose
            tokens will be treated as documents. Default is `sentence`.
        :type context_type: string, optional

        :param psi:  
        :type psi: int, optional

        :param rand_perm:  
        :type rand_perm: boolean, optional

        :param lmda:  
        :type lmda: int, optional
        """
        self.context_type = context_type
        self.sents = corpus.view_contexts(context_type)
        self.env_matrix = env_matrix

        self.b_conv = mk_b_conv(env_matrix.shape[1], rand_perm)
        
        if psi is None:
            self.psi = rand_pt_unit_sphere(env_matrix.shape[1])
        else:
            self.psi = psi

        self.lmda = lmda


    def train(self):
        """
        Trains the model.
        """
        self.matrix = np.zeros_like(self.env_matrix)

        for sent in self.sents:
            env = self.env_matrix[sent]
            
            for i in xrange(sent.size):
                sent_vecs = env.copy() 
                sent_vecs[i, :] = self.psi[:]

                conv_ngrams = reduce_ngrams(self.b_conv, sent_vecs, self.lmda, i)
                    
                ord_vec = np.sum(conv_ngrams, axis=0)
                self.matrix[sent[i], :] += ord_vec



class BeagleOrderMulti(BaseModel):
    """
    `BeagleOrderSeq` stores word order information in the context.
    """
    def __init__(self, corpus, env_matrix, context_type='sentence',
                 psi=None, rand_perm=None, lmda =7):
        """    
        Initialize BeagleOrderMulti.

        :param corpus: Source of observed data.
        :type corpus: Corpus

        :param env_matrix: BEAGLE environement matrix.
        :type env_matrix: 2-D array

        :param context_type: Name of tokenization stored in `corpus` whose
            tokens will be treated as documents. Default is `sentence`.
        :type context_type: string, optional

        :param psi:  
        :type psi: int, optional

        :param rand_perm:  
        :type rand_perm: boolean, optional

        :param lmda:  
        :type lmda: int, optional
        """
        self.context_type = context_type
        self.sents = corpus.view_contexts(context_type)
        self.dtype = env_matrix.dtype

        global _shape 
        _shape = mp.Array('i', 2, lock=False)
        _shape[:] = env_matrix.shape

        print 'Copying env matrix to shared mp array'
        global _env_matrix
        _env_matrix = mp.Array('d', env_matrix.size, lock=False)
        _env_matrix[:] = env_matrix.ravel()[:]

        #TODO: Convert this to a shared data structure
        global _b_conv 
        _b_conv = mk_b_conv(env_matrix.shape[1], rand_perm)

        global _psi        
        _psi = mp.Array('d', _shape[1], lock=False)
        if psi is None:
            _psi[:] = rand_pt_unit_sphere(env_matrix.shape[1])[:]
        else:
            _psi[:] = psi[:]

        global _lmda
        _lmda = mp.Value('i', lock=False)
        _lmda =lmda


    def train(self, n_procs=2):
        """
        Trains the model using `n_procs` processors.

        :param n_procs: Number of processors. Default is 2.
        :type n_procs: int, optional
        """
        sent_lists = np.array_split(self.sents, n_procs-1)
        if len(sent_lists) != n_procs:
            sent_lists = np.array_split(self.sents, n_procs)

        tmp_dir = tempfile.mkdtemp()
        tmp_files = [os.path.join(tmp_dir, 'tmp_' + str(i))
                     for i in xrange(len(sent_lists))]

        sent_lists = zip(sent_lists, tmp_files)
        del self.sents


        try:
            print 'Forking'
            # For debugging
            # tmp_files = map(mpfn, sent_lists)
            
            p = mp.Pool(n_procs)
            tmp_files = p.map(mpfn, sent_lists, 1)
            p.close()

            print 'Reducing'
            self.matrix = np.zeros(tuple(_shape), dtype=self.dtype)

            for filename in tmp_files:

                with open(filename, 'rb') as f:
                    result = cpickle.load(f)

                for k,v in result.iteritems():
                    self.matrix[k, :] += v

        finally:
            print 'Removing', tmp_dir
            shutil.rmtree(tmp_dir)
        

def mpfn((sents, filename)):
    """
    """
    result = dict()

    for sent in sents:

        env = np.empty((sent.size, _shape[1]), dtype=np.float64)
        for i,w in enumerate(sent):
            env[i, :] = _env_matrix[w*_shape[1]: (w+1)*_shape[1]]

        for i,w in enumerate(sent):
            sent_vecs = env.copy()
            sent_vecs[i, :] = _psi[:]

            conv_ngrams = reduce_ngrams(_b_conv, sent_vecs, _lmda, i)

            ord_vec = np.sum(conv_ngrams, axis=0)

            if w in result:
                result[w] += ord_vec
            else:
                result[w] = ord_vec
            
    with open(filename, 'wb') as f:
        cpickle.dump(result, f)

    return filename



from __future__ import unicode_literals

import numpy as np

from vsm.structarr import *
from types import *


__all__ = ['DataTable', 'IndexedSymmArray', 'LabeledColumn']



def format_entry(x):
    """
    Formats floats to 5 decimal points and returns a string.
    If `x` is a tuple, all elements in the tuple are formatted.

    :param x: Float to be truncated to 5 decimal points.
    :type x: float or tuple

    :returns: `x` as a string.
    """
    # np.void is the type of the tuples that appear in numpy
    # structured arrays
    if isinstance(x, np.void):
        return ', '.join([format_entry(i) for i in x.tolist()]) 
        
    if isfloat(x):
        return '{0:.5f}'.format(x)

    return str(x)


def format_(x, n):
    """
    Truncates `x` given the length `n`. If `x` is a float 
    it returns floats formatted to 5 decimal points.

    :param x: Element to be truncated.
    :type x: string or integer/float
    
    :param n: Length that we want `x` to be.
    :type n: integer

    :returns: formatted `x` given the length `n`.
    """
    if isint(x):
        return x

    if isfloat(x):
        return '{0:.5f}'.format(x)

    n = min(n, len(x))
    return x[:n]


def default_col_widths(dtype, col_header):
    """
    Assigns default column widths depending on the dtype. 
    Used in _str_ representation.
    """
    col_widths = []
    
    values =zip(*dtype.fields.values())[0]

    for t in values:
        if t.kind == 'S':
            col_widths.append(t.itemsize + 5)
        else:
            col_widths.append(len(col_header) + 5)
    
    return col_widths


def calc_col_num(col_len, n):
    
    num = col_len / n

    if col_len % n > 0:
        num += 1

    return num


def max_col_num(li, max_width):
    """
    Calculates the total number of columns for multi_col option
    in LabeledColumn.
    """
    w = sum(li)
    num = max_width/w
    if num == 0:
        return 1
    return num


class LabeledColumn(np.ndarray):
    """
    A subclass of np.ndarray whose purpose is to store labels and
    formatting information for a 1-dimensional structured array. It
    also provides pretty-printing routines.

    A column can have a header and a default display length.

    A subcolumn wraps the data found under a given field name. Each
    subcolumn has a label and a display width.

    :param input_array: Array to be formatted into a LabeledColumn.
    :type input_array: 1-dimensional structured array

    :param col_header: The title of the object. For example, 'Words: logic'.
        Default is `None`.
    :type col_header: string, optional
    
    :param subcol_headers: List of labels that correspond to the fields of 
        the structured array. Default is `None`.
    :type subcol_headers: list, optional

    :param subcol_widths: List of widths for each subcolumn. If not provided, 
        `subcol_widths` is calculated based on the data-type of the entries.
    :type subcol_widths: list, optional
    
    :param col_len: Number of entries to display. If not provided, `col_len`
        is set to length of LabeledColumn.
    :type col_len: integer, optional

    :param multi_col: If `True` html table version of :class:`LabeledColumn`
        is displayed in multiple columns when the number of entries exceed 15.
        Default is `True`.
    :type multi_col: boolean, optional
        
    :attributes:
        * **col_header** (string, optional)
            The title of the object. For example, 'Words: logic'.
            Default is `None`.
        * **subcol_headers** (list, optional)
            List of labels that correspond to the fields of 
            the structured array. Default is `None`.
        * **subcol_widths** (list, optional)
            List of widths for each subcolumn. If not provided, 
            `subcol_widths` is calculated based on the data-type of the entries.
    
        * **col_len** (integer, optional) 
            Number of entries to display. If not provided, `col_len`
            is set to length of LabeledColumn.

    :methods:
        * **__str__**
            Returns a pretty printed string version of the object.
        * **_repr_html_**
            Returns a html table in ipython online session.

    **Examples**
    
    >>>  words = ['there','will','be','an','answer']
    >>>  values = [random.random() for w in words]
    >>>  arr = np.array(zip(words, values), 
            dtype=[('i', np.array(words).dtype), 
            ('value', np.array(values).dtype)])
    >>>  lc = LabeledColumn(arr)
    >>>  lc.col_header = 'Words'
    >>>  lc.subcol_headers = ['Word', 'Value']
    >>>  lc.subcol_widths
    [11, 10]
    >>>  lc.col_len
    5
    >>> print lc
    Words        
    ---------------------
    Word       Value     
    ---------------------
    there      0.22608   
    will       0.64567   
    be         0.02832   
    an         0.31118   
    answer     0.23083   

    """
    def __new__(cls, input_array, col_header=None, subcol_headers=None,
             subcol_widths=None, col_len=None, col_num=None, multi_col=True):
        """
        """
        obj = np.asarray(input_array).view(cls)
        obj.col_header = col_header
        obj._col_len = col_len
        obj.subcol_headers = subcol_headers
        obj._subcol_widths = subcol_widths     
        obj._col_num = col_num
        obj.multi_col = multi_col

        return obj


    def __array_finalize__(self, obj):
        """
        """
        if obj is None: return

        self.col_header = getattr(obj, 'col_header', None)
        self._col_len = getattr(obj, '_col_len', None)
        self.subcol_headers = getattr(obj, 'subcol_headers', None)
        self._subcol_widths = getattr(obj, '_subcol_widths', None)
        self._col_num = getattr(obj, '_col_num', None)
        self.multi_col = getattr(obj, 'multi_col', True)

    @property
    def col_num(self):
        if not hasattr(self, '_col_num') or not self._col_num:
            self._col_num = min(calc_col_num(self._col_len, 15),
                                max_col_num(self._subcol_widths, 160))
        return self._col_num

    @col_num.setter
    def col_num(self, w):
        self._col_num = w

    @property
    def subcol_widths(self):
        if not hasattr(self, '_subcol_widths') or not self._subcol_widths:
            self._subcol_widths = default_col_widths(self.dtype, self.col_header)
        return self._subcol_widths

    @subcol_widths.setter
    def subcol_widths(self, w):
        self._subcol_widths = w

    @property
    def col_width(self):
        return max(sum(self.subcol_widths), len(self.col_header))

    @property
    def col_len(self):
        if not self._col_len:
            return self.shape[0]
        return min(self.shape[0], self._col_len)

    @col_len.setter
    def col_len(self, n):
        self._col_len = n


    def __str__(self):
        """
        Pretty prints the LabeledColumn when 'print' method is used.
        """     
        line = '-' * self.col_width + '\n'
        out = line
        if self.col_header:
            out += '{0:^{1}}'.format(format_(self.col_header, self.col_width), 
                                     self.col_width) + '\n'
            out += line
            
        if self.subcol_headers:
            for i in xrange(len(self.subcol_headers)):
                w = self.subcol_widths[i]
                out += '{0:<{1}}'.format(format_(self.subcol_headers[i], w), w)
            out += '\n'
            out += line

        for i in xrange(self.col_len):
            for j in xrange(len(self.dtype)):
                w = self.subcol_widths[j]
                n = self.dtype.names[j]
                out += '{0:<{1}}'.format(format_(self[n][i], w), w)
            out += '\n'

        return out


    def _repr_html_(self):
        """
        Returns an html table in ipython online session.
        """ 
        s = '<table style="margin: 0">'
        if self.multi_col and self.col_len >15:
            if self.col_header:
                s += '<tr><th style="text-align: center; background: #CEE3F6" colspan\
                    ="{0}">{1}</th></tr>'.format(len(self.subcol_widths)*self.col_num, 
                    self.col_header)
            
            if self.subcol_headers:
                s += '<tr>'
                subcol = ''
                for sch in self.subcol_headers:
                    subcol += '<th style="text-align: center; background: #EFF2FB; ">{0}\
                    </th>'.format(sch)
                s += subcol * self.col_num
                s += '</tr>'
            
            count = self.col_len
            last_row = self.col_len % self.col_num
            rows = self.col_len / self.col_num
            
            li = [rows] * self.col_num
            li = [li[i]+1 if i<last_row else li[i] for i in xrange(self.col_num)]
            li = [0] + li[:-1]
            if last_row > 0:
                rows += 1            

            for k in xrange(rows):
                s += '<tr>'
                ind = k
                for i in xrange(self.col_num):
                    ind += li[i]
                    for j in xrange(len(self.dtype)):
                        w = self.subcol_widths[j]
                        n = self.dtype.names[j]
                        if count > 0:
                            s += '<td>{0:<{1}}</td>'.format(format_(self[n][ind],w), w)
                        else:
                            s += '<td style="border-color: #EFF2FB; background: #EEF2FB;">\
                             {0}</td>'.format(' '* w)
                    count -= 1    
                s += '</tr>'
        else:
            if self.col_header:
                s += '<tr><th style="text-align: center; background: #CEE3F6" colspan\
                    ="{0}">{1}</th></tr>'.format(len(self.subcol_widths), self.col_header)

            if self.subcol_headers:
                s += '<tr>'
                for sch in self.subcol_headers:
                    s += '<th style="text-align: center; background: #EFF2FB; ">{0}\
                          </th>'.format(sch)
                s += '</tr>'
        
            for i in xrange(self.col_len):
                s += '<tr>'
                for j in xrange(len(self.dtype)):
                    w = self.subcol_widths[j]
                    n = self.dtype.names[j]
                    s += '<td>{0:<{1}}</td>'.format(format_(self[n][i], w), w)
                s += '</tr>'
        
        s += '</table>'
        return s


class DataTable(list):
    """
    A subclass of list whose purpose is to store labels and
    formatting information for a list of 1-dimensional structured
    arrays. It also provides pretty-printing routines.

    Globally, the table has a default display length for the columns
    and a table header.

    A column can have a column-specific header.

    A subcolumn wraps the data found under a given field name. Each
    subcolumn has a label and a display width.
 
    :param l: List of 1-dimensional structured arrays.
    :type l: list
    
    :param table_header: The title of the object. Default is `None`.
    :type table_header: string, optional
  
    :param compact_view: If `True` the DataTable is displayed with its
        tokens only without the probabilities. Default is `True`
    :type compact_view: boolean, optional

    :attributes:
        * **table_header** (string)
            The title of the object. Default is `None`.
        * **compact_view** (boolean)
            Option of viewing tokens with or without the probabilities.
    :methods:
        * **__str__**
            Returns a pretty printed string version of the object.
        * **_repr_html_**
            Returns an html table in ipython online session.

    **Examples**
    
    >>>  words = ['there','will','be','an','answer']
    >>>  values = [random.random() for w in words]
    >>>  arr = np.array(zip(words, values), 
            dtype=[('i', np.array(words).dtype), 
            ('value', np.array(values).dtype)])
    >>>  lc = LabeledColumn(arr, 'Lyrics')
    >>>  l = [lc.copy() for i in xrange(2)]
    >>>  dt = DataTable(l, 'Let it be', subcolhdr_compact=['Topic', 'Words'],
                   subcolhdr_full=['Word', 'Prob'], compact_view=True)
    >>>  print dt
    --------------------------------------------
                     Let it be                  
    --------------------------------------------
    Topic      Words      
    --------------------------------------------
    Lyrics     there      will       be         
               an         answer     
    --------------------------------------------
    Lyrics     there      will       be         
               an         answer     
    --------------------------------------------

    >>> dt.compact_view = False
    >>>  print dt
        Let it be      
    ---------------------
            Words        
    ---------------------
    Word       Value     
    ---------------------
    there      0.58793   
    will       0.29624   
    be         0.00209   
    an         0.27221   
    answer     0.96118   
    ---------------------
            Words        
    ---------------------
    Word       Value     
    ---------------------
    there      0.22608   
    will       0.64567   
    be         0.02832   
    an         0.31118   
    answer     0.23083      
    """
    
    def __init__(self, l, table_header=None, compact_view=True,
        subcolhdr_compact=None, subcolhdr_full=None):
        """
        """
        super(DataTable, self).__init__(l)
        self.table_header = table_header
        self.compact_view = compact_view
        self.subcolhdr_compact = subcolhdr_compact
        self.subcolhdr_full = subcolhdr_full

    
    def __getslice__(self, i, j):
        """
        """
        return DataTable(list.__getslice__(self, i, j), 
                    table_header=self.table_header, 
                    compact_view=self.compact_view,
                    subcolhdr_compact=self.subcolhdr_compact,
                    subcolhdr_full=self.subcolhdr_full)
                
    def __str__(self):
        """
        Pretty prints the DataTable when `print` method is used.
        """
        if self.compact_view:
            return self.__str_compact__(self.subcolhdr_compact)
        else:
            return self.__str_full__(self.subcolhdr_full)


    def __str_compact__(self, subcol_headers):
        """
        Prints DataTable when `compact_view` is `True`.
        """
        num_cols = 3
        w1 = self[0].subcol_widths[1]
        w2 = min(self[0].subcol_widths[0], 23)
        col_width = w1 + w2 * 3  
        
        line = '-' * col_width + '\n'
        out = line
        if self.table_header:
            out += '{0:^{1}}'.format(format_(self.table_header, col_width), 
                                     col_width) + '\n'
            out += line
        
        if subcol_headers:
            out += '{0:<{1}}'.format(format_(subcol_headers[0], w1), w1)  
            out += '{0:<{1}}\n'.format(format_(subcol_headers[1], w2), w2)
            out += line
        
        for i in xrange(len(self)):
            j = 0
            # topic info
            out += '{0:<{1}}'.format(format_(self[i].col_header, w1), w1)
            # words or tokens
            for idx in xrange(self[i].col_len):
                if j == 0 and idx > 0 :
                    out += " " * w1
                word = self[i].dtype.names[0]
                out += '{0:<{1}}'.format(format_(self[i][word][idx], w2-1), w2)
                j += 1
                if j == num_cols or idx == self[i].col_len -1:
                    out += '\n'
                    j = 0
            out += line
        
        return out
    

    def __str_full__(self, subcol_headers):
        """
        Prints DataTable when `compact_view` is `False`.
        """
        col_width = self[0].col_width

        out = '-' * col_width + '\n'
        if self.table_header:
            out += '{0:^{1}}'.format(format_(self.table_header, col_width), 
                                     col_width) + '\n'

        for col in self:
            col.subcol_headers = subcol_headers
            out += col.__str__()

        return out


    def _repr_html_(self):
        """
        Returns a html table in ipython online session. 
        """
        if self.compact_view:    
            return self._repr_html_compact_(self.subcolhdr_compact)
        else:
            return self._repr_html_full_(self.subcolhdr_full)


    def _repr_html_compact_(self, subcol_headers):
        """
        Returns a html table in ipython online session when 
        `compact_view` is `True`.
        """
        s = '<table style="margin: 0">'

        num_cols = 3
        if self.table_header:
            s += '<tr><th style="text-align: center; background: #CEE3F6" colspan\
            ="{0}">{1}</th></tr>'.format(1 + self[0].col_len, self.table_header)
           
        s += '<tr>'
        for i, sch in enumerate(subcol_headers):
            s += '<th style="text-align: center; background: #EFF2FB;" \
                 >{0}</th>'.format(sch)
        s += '</tr>'
        
        for i in xrange(len(self)):
            s += '<tr>'
            w = self[0].subcol_widths[1]
            s += '<td style="padding-left:0.75em;">{0}</td>'.format(
                    format_(self[i].col_header, w), w)
# line break.
            s += '<td>' 
            for j in xrange(self[0].col_len):
                n = self[0].dtype.names[0] 
                w = self[0].subcol_widths[0]
                if j == self[0].col_len -1:
                    s += ' {0:<{1}}'.format(self[i][n][j], w)
                else:
                    s += ' {0},'.format(self[i][n][j])
            s += '</td>'
            s += '</tr>'
        s += '</table>'
 
        return s

    
    def _repr_html_full_(self, subcol_headers):
        """
        Returns a html table in ipython online session when
        `compact_view` is `False`.
        """        
        s = '<table>'

        col_in_row = 3
        
        n_cols = len(subcol_headers)
        col_w = self[0].col_width

        if self.table_header:
            s += '<tr><th style="text-align: center; background: #A9D0F5;\
            fontsize: 14px;" colspan="{0}"> {1} </th></tr>'.format(col_in_row
             * n_cols, self.table_header)
     
        start = 0
        n_arr = len(self)
        m = n_arr % col_in_row

        while start < n_arr:
            end = start + col_in_row
            group = self[start:end]     
        
            s += '<tr>'
            for i, lc in enumerate(group):
                if lc.col_header:
                    s += '<th style="text-align: center; background: #CEE3F6;"\
                 colspan="{0}">{1}</th>'.format(n_cols, lc.col_header)

                if end > n_arr and m and i == len(group)-1 and start > 0:
                    for j in xrange(end - n_arr):
                        s += '<th style="border-color: #EFF2FB; background: #EFF2FB;"\
                        colspan="{0}"> {1}</th>'.format(n_cols, ' ' * col_w)
            s += '</tr>'

            s += '<tr>'
            for i, lc in enumerate(group):
                    
                for sch in subcol_headers:
                    s += '<th style="text-align: center; background: #EFF2FB;">\
                        {0}</th>'.format(sch)

                if end > n_arr and m and i == len(group)-1 and start > 0:
                    for j in xrange(end - n_arr):
                        s += '<th style="border-color: #EFF2FB; background: #EFF2FB;"\
                         colspan="{0}"> {1}</th>'.format(n_cols, ' ' * col_w)
            s += '</tr>'
            
            for i in xrange(self[0].col_len):
    
                s += '<tr>'
                for k, lc in enumerate(group):
                  
                    for j in xrange(len(lc.dtype)):
                        w = lc.subcol_widths[j]
                        n = lc.dtype.names[j]
                        s += '<td>{0}</td>'.format(format_(lc[n][i], w))
    
                    if end > n_arr and m and k == len(group)-1 and start > 0:
                        for e in xrange(end - n_arr):
                            s += '<td style="border-color: #EFF2FB; background: #EFF2FB;"\
                            colspan="{0}"> {1} </th>'.format(n_cols, ' ' * col_w)
                s += '</tr>'    
            
            start = end
        s += '</table>'

        return s


class IndexedSymmArray(np.ndarray):
    """
    """
    def __new__(cls, input_array, labels=None):
        """
        """
        obj = np.asarray(input_array).view(cls)
        obj.labels = labels

        return obj


    def __array_finalize__(self, obj):
        """
        """
        if obj is None: return
        self.labels = getattr(obj, 'labels', None)



import re
import string
import platform

from chardet.universaldetector import UniversalDetector
import nltk
import numpy as np

__all__ = ['strip_punc', 'rem_num', 'rehyph',
           'apply_stoplist', 'filter_by_suffix', 'word_tokenize',
           'sentence_tokenize', 'paragraph_tokenize', 'detect_encoding',
           'sentence_span_tokenize', 'in_place_stoplist']



def strip_punc(tsent):
    """
    """
    p1 = re.compile(ur'^([{}\u0000-\u0020\u2000-\u206F\u3000-\u303F\uFF00-\uFFFF]*)'.format(string.punctuation))
    p2 = re.compile(ur'([{}\u0000-\u0020\u2000-\u206F\u3000-\u303F\uFF00-\uFFFF]*)$'.format(string.punctuation))

    out = []
    for word in tsent:
        w = re.sub(p2, '', re.sub(p1, '', word))
        if w:
            out.append(w)

    return out


def rem_num(tsent):
    """
    """
    p = re.compile(r'(^\D+$)|(^\D*[1-2]\d\D*$|^\D*\d\D*$)')

    return [word for word in tsent if re.search(p, word)]


def rehyph(sent):
    """
    """
    return re.sub(r'(?P<x1>.)--(?P<x2>.)', '\g<x1> - \g<x2>', sent)


def apply_stoplist(corp, nltk_stop=True, add_stop=None, freq=0, in_place=True):
    """
    Returns a Corpus object with stop words eliminated.

    :param corp: Corpus object to apply stoplist to.
    :type corp: Corpus

    :param nltk_stop: If `True` English stopwords from nltk are included
        in the stoplist. Default is `True`.
    :type nltk_stop: boolean, optional
    
    :param add_stop: list of words to eliminate from `corp` words.
        Default is `None`.
    :type add_stop: List, optional

    :param freq: Eliminates words that appear <= `freq` times. Default is
        0.
    :type freq: int

    :returns: Corpus with words in the stoplist removed.

    :See Also: :class:`vsm.corpus.Corpus`, :meth:`vsm.corpus.Corpus.apply_stoplist`
    """
    stoplist = set()
    if nltk_stop:
        for w in nltk.corpus.stopwords.words('english'):
            stoplist.add(w)
    if add_stop:
        for w in add_stop:
            stoplist.add(w)

    if not in_place:
        return corp.apply_stoplist(stoplist=stoplist, freq=freq)
    else:
        corp.in_place_stoplist(stoplist=stoplist, freq=freq)
        return corp

def in_place_stoplist(corp, nltk_stop=True, add_stop=None, freq=0):
    """
    Returns a Corpus object with stop words eliminated.

    :param corp: Corpus object to apply stoplist to.
    :type corp: Corpus

    :param nltk_stop: If `True` English stopwords from nltk are included
        in the stoplist. Default is `True`.
    :type nltk_stop: boolean, optional
    
    :param add_stop: list of words to eliminate from `corp` words.
        Default is `None`.
    :type add_stop: List, optional

    :param freq: Eliminates words that appear <= `freq` times. Default is
        0.
    :type freq: int

    :returns: Corpus with words in the stoplist removed.

    :See Also: :class:`vsm.corpus.Corpus`, :meth:`vsm.corpus.Corpus.apply_stoplist`
    """
    stoplist = set()
    if nltk_stop:
        for w in nltk.corpus.stopwords.words('english'):
            stoplist.add(w)
    if add_stop:
        for w in add_stop:
            stoplist.add(w)

    corp.in_place_stoplist(stoplist=stoplist, freq=freq)


def filter_by_suffix(l, ignore, filter_dotfiles=True):
    """
    Returns elements in `l` that does not end with elements in `ignore`,
    and filters dotfiles (files that start with '.').

    :param l: List of strings to filter.
    :type l: list

    :param ignore: List of suffix to be ignored or filtered out.
    :type ignore: list

    :param filter_dotfiles: Filter dotfiles.
    :type filter_dotfiles: boolean, default True

    :returns: List of elements in `l` whose suffix is not in `ignore`.

    **Examples**

    >>> l = ['a.txt', 'b.json', 'c.txt']
    >>> ignore = ['.txt']
    >>> filter_by_suffix(l, ignore)
    ['b.json']
    """
    if platform.system() == 'Windows':
        import win32api
        l = [win32api.GetShortPathName(filename) for filename in l]
    filter_list = [e for e in l if not sum([e.endswith(s) for s in ignore])]
    if filter_dotfiles:
        filter_list = [e for e in filter_list if not e.startswith('.')]
    return filter_list


def word_tokenize(text):
    """Takes a string and returns a list of strings. Intended use: the
    input string is English text and the output consists of the
    lower-case words in this text with numbers and punctuation, except
    for hyphens, removed.

    The core work is done by NLTK's Treebank Word Tokenizer.
    
    :param text: Text to be tokeized.
    :type text: string

    :returns: tokens : list of strings
    """

    text = rehyph(text)
    text = nltk.TreebankWordTokenizer().tokenize(text)

    tokens = [word.lower() for word in text]
    tokens = strip_punc(tokens)
    tokens = rem_num(tokens)
    
    return tokens


def sentence_tokenize(text):
    """
    Takes a string and returns a list of strings. Intended use: the
    input string is English text and the output consists of the
    sentences in this text.

    This is a wrapper for NLTK's pre-trained Punkt Tokenizer.
     
    :param text: Text to be tokeized.
    :type text: string

    :returns: tokens : list of strings
    """
    tokenizer = nltk.data.load('tokenizers/punkt/english.pickle')

    return tokenizer.tokenize(text)

def sentence_span_tokenize(text):
    """
    Takes a string and returns a list of strings. Intended use: the
    input string is English text and the output consists of the
    sentences in this text.

    This is a wrapper for NLTK's pre-trained Punkt Tokenizer.
     
    :param text: Text to be tokeized.
    :type text: string

    :returns: token_spans : iterator of (start,stop) tuples.
    """
    tokenizer = nltk.data.load('tokenizers/punkt/english.pickle')

    return tokenizer.span_tokenize(text)



def paragraph_tokenize(text):
    """
    Takes a string and returns a list of strings. Intended use: the
    input string is English text and the output consists of the
    paragraphs in this text. It's expected that the text marks
    paragraphs with two consecutive line breaks.
     
    :param text: Text to be tokeized.
    :type text: string

    :returns: tokens : list of strings
    """

    par_break = re.compile(r'[\r\n]{2,}')
    span_start = 0
    for match in par_break.finditer(text):
        span_end = match.span()[0]
        yield text[span_start:span_end]
        span_start = match.span()[1]
    yield text[span_start:]
    #return par_break.split(text)


def detect_encoding(filename):
    """
    Takes a filename and attempts to detect the character encoding of the file
    using `chardet`.
     
    :param filename: Name of the file to process
    :type filename: string

    :returns: encoding : string
    """
    detector = UniversalDetector()
    with open(filename, 'rb') as unknown_file:
        for line in unknown_file:
            detector.feed(line)
            if detector.done:
                break
    detector.close()

    return detector.result['encoding']
     



import warnings
from functools import update_wrapper, wraps
import inspect

__all__ = ['deprecation_warning', 'deprecated_meth']



def deprecation_warning(old_name, new_name):
    """
    Deprecation warning for deprecated functions.
    """
    warnings.simplefilter('always', DeprecationWarning)
    
    message = "{0} is deprecated. Please use {1} instead.".format(old_name,
                new_name)
    warnings.warn(message, DeprecationWarning)


#TODO: a function for deprecated class AND auto generate doc string with
# a note about deprecation.

def deprecated_meth(new_fn_name):
    """
    Decorator to be used for deprecated functions/modules.
    Throws a DeprecationWarning.
    """
    def wrap(old_fn):
        
        def wrapper(self, *args, **kwargs):
            new_fn = getattr(self, new_fn_name)
            deprecation_warning(old_fn.__name__, new_fn.__name__)
            
            return new_fn(*args, **kwargs)
  
        #update_wrapper(wrapper, new_fn_)
        return wrapper

    return wrap


"""
def deprecated_fn(new_fn):
    Decorator to be used for deprecated functions/modules.
    Throws a DeprecationWarning.
    def wrap(old_fn):
        
        def wrapper(self, *args, **kwargs):
            deprecation_warning(old_fn.__name__, new_fn.__name__)
            #new_fn_ = getattr(self, new_fn.__name__)

            return new_fn(self, *args, **kwargs)
  
        update_wrapper(wrapper, new_fn)
        
        return wrapper

    return wrap
"""



import os
import shutil
import tempfile
import multiprocessing as mp
import cPickle as cpickle

import numpy as np

from base import BaseModel


__all__ = [ 'BeagleContextSeq', 'BeagleContextMulti' ]


def realign_env_mat(corpus, env_corpus, env_matrix):
    """
    """
    words = corpus.words
    indices = [env_corpus.words_int[w] for w in words]
    return env_matrix[indices]



class BeagleContextSeq(BaseModel):
    """

    """
    def __init__(self, corpus, env_corpus, env_matrix, 
                context_type='sentence'):
        """
        Initialize BeagleContextSeq.

        :param corpus: Source of observed data.
        :type corpus: class:`Corpus`

        :param env_corpus: BEAGLE environment corpus.
        :type env_corpus: class:`Corpus`

        :param env_matrix: Matrix from BEAGLE environment model.
        :type env_matrix: 2-D array

        :param context_type: Name of tokenization stored in `corpus` whose
            tokens will be treated as documents. Default is `sentence`.
        :type context_type: string, optional
        """
        self.context_type = context_type
        self.sents = corpus.view_contexts(context_type)
        self.env_matrix = realign_env_mat(corpus, env_corpus, env_matrix)


    def train(self):
        """
        Trains the model.
        """
        self.matrix = np.zeros_like(self.env_matrix)

        for sent in self.sents:

            if sent.shape[0] > 1:

                left_sums = np.cumsum(self.env_matrix[sent[:-1]], axis=0)
                right_sums = np.cumsum(self.env_matrix[sent[:0:-1]], axis=0)

                for i,word in enumerate(sent):

                    if i == 0:
                        ctx_vector = right_sums[-1]

                    elif i == sent.shape[0] - 1:
                        ctx_vector = left_sums[-1]
                    
                    else:
                        ctx_vector = left_sums[i - 1] + right_sums[-i - 1]

                    self.matrix[word, :] += ctx_vector



class BeagleContextMulti(BaseModel):
    """

    """

    def __init__(self, corpus, env_corpus, env_matrix, 
                 context_type='sentence'):
        """
        Initialize BeagleContextMulti.
        
        :param corpus: Souce of observed data.
        :type corpus: class:`Corpus`

        :param env_corpus: BEAGLE environment corpus. 
        :type env_corpus: class:`Corpus`

        :param env_matrix: Matrix from BEAGLE environment model.
        :type env_matrix: 2-D array

        :param context_type: Name of tokenization stored in `corpus` whose
            tokens will be treated as documents. Default is `sentence`.
        :type context_type: string, optional
        """
        self.context_type = context_type
        self.sents = corpus.view_contexts(context_type)
        self.dtype = env_matrix.dtype
        env_matrix = realign_env_mat(corpus, env_corpus, env_matrix)

        global _shape 
        _shape = mp.Array('i', 2, lock=False)
        _shape[:] = env_matrix.shape
        
        print 'Copying env matrix to shared mp array'
        global _env_matrix
        _env_matrix = mp.Array('d', env_matrix.size, lock=False)
        _env_matrix[:] = env_matrix.ravel()[:]


    def train(self, n_procs=2):
        """
        Takes an optional argument `n_procs`, number of processors,
        and trains the model on the number of processors. `n_procs`
        is 2 by default.
        
        :param n_procs: Number of processors. Default is 2.
        :type n_procs: int, optional

        :returs: `None`
        """
        sent_lists = np.array_split(self.sents, n_procs-1)
        if len(sent_lists) != n_procs:
            sent_lists = np.array_split(self.sents, n_procs)

        tmp_dir = tempfile.mkdtemp()
        tmp_files = [os.path.join(tmp_dir, 'tmp_' + str(i))
                     for i in xrange(len(sent_lists))]

        sent_lists = zip(sent_lists, tmp_files)
        del self.sents

        try:
            print 'Forking'
            # For debugging
            # tmp_files = map(mpfn, sent_lists)
            
            p = mp.Pool(n_procs)
            tmp_files = p.map(mpfn, sent_lists, 1)
            p.close()

            print 'Reducing'
            self.matrix = np.zeros(tuple(_shape), dtype=self.dtype)

            for filename in tmp_files:

                with open(filename, 'rb') as f:
                    result = cpickle.load(f)

                for k,v in result.iteritems():
                    self.matrix[k, :] += v

        finally:
            print 'Removing', tmp_dir
            shutil.rmtree(tmp_dir)



def mpfn((sents, filename)):
    """
    """
    result = dict()

    for sent in sents:
        if sent.shape[0] > 1:

            env = np.empty((sent.size, _shape[1]), dtype=np.float64)
            for i,w in enumerate(sent):
                env[i, :] = _env_matrix[w*_shape[1]: (w+1)*_shape[1]]
        
            left_sums = np.cumsum(env[:-1], axis=0)
            right_sums = np.cumsum(env[:0:-1], axis=0)

            for i,t in enumerate(sent):
                
                if i == 0:
                    ctx_vector = right_sums[-1]

                elif i == sent.shape[0] - 1:
                    ctx_vector = left_sums[-1]
                    
                else:
                    ctx_vector = left_sums[i - 1] + right_sums[-i - 1]
                   
                if t in result:
                    result[t] += ctx_vector
                else:
                    result[t] = ctx_vector

    with open(filename, 'wb') as f:
        cpickle.dump(result, f)

    return filename




"""
Provides the class `LdaCgsViewer`.
"""

import numpy as np

from vsm.spatial import H, JS_dist, KL_div
from vsm.structarr import *
from vsm.split import split_corpus
from vsm.exceptions import *
from types import *
from labeleddata import *
from wrappers import *


__all__ = [ 'LdaCgsViewer' ]


class LdaCgsViewer(object):
    """A class for viewing a topic model estimated by one of vsm's LDA
    classes using CGS.
    """
    
    def __init__(self, corpus, model):
        """
        Initialize LdaCgsViewer.

        :param corpus: Source of observed data.
        :type corpus: :class:`Corpus`
    
        :param model: An LDA model estimated by a CGS.
        :type model: LdaCgsSeq
        """
        self.corpus = corpus
        self.model = model
        self._phi = None
        self._theta = None
        self._H_phi = None
        self._H_theta = None
        self._labels = None


    @property
    def _doc_label_name(self):
        return doc_label_name(self.model.context_type)


    def _res_doc_type(self, doc):
        return res_doc_type(self.corpus, self.model.context_type, 
                              self._doc_label_name, doc)


    def _res_word_type(self, word):
        return res_word_type(self.corpus, word)

    @property
    def labels(self):
        """Returns the list of document labels."""
        if self._labels is None:
            self._labels = self.corpus.view_metadata(self.model.context_type)
            self._labels = self._labels[self._doc_label_name]

        return self._labels


    @property
    def phi(self):
        """Returns the word by topic matrix from the model as a right
        stochastic matrix (the columns phi_i are probability
        distributions).

        """
        if self._phi is None:
            self._phi = self.model.word_top / self.model.word_top.sum(0)
        return self._phi


    @property
    def theta(self):
        """Returns the topic by document matrix from the model as a right
        stochastic matrix (the columns theta_i are probability
        distributions.

        """
        if self._theta is None:
            self._theta =  self.model.top_doc / self.model.top_doc.sum(0)
        return self._theta


    @property
    def H_phi(self):
        """Returns the entropies of the columns of phi (i.e., topics)

        """
        if self._H_phi is None:
            self._H_phi = H(self.phi.T)
        return self._H_phi


    @property
    def H_theta(self):
        """Returns the entropies of the columns of theta.

        """
        if self._H_theta is None:
            self._H_theta = H(self.theta.T)
        return self._H_theta


    def topic_entropies(self, print_len=10):
        """Returns the entropies of the topics of the model as an array sorted
        by entropy.

        """
        H_phi = self.H_phi
        k_arr = enum_sort(H_phi).view(LabeledColumn)
        k_arr.col_header = 'Topic Entropies'
        k_arr.subcol_headers = ['Index', 'Entropy']
        k_arr.col_len = print_len

        return k_arr[::-1]


    def doc_entropies(self, print_len=10, 
                      label_fn=def_label_fn, as_strings=True):
        """Returns the entropies of the distributions over topics as an
        array sorted by entropy.

        """
        if as_strings:
            md = self.corpus.view_metadata(self.model.context_type)
            docs = label_fn(md)
            d_arr = enum_sort(self.H_theta, indices=docs, field_name='doc')
        else:
            d_arr = enum_sort(self.H_theta)

        d_arr = d_arr.view(LabeledColumn)
        d_arr.col_header = 'Document Entropies'
        d_arr.subcol_headers = ['Document', 'Entropy']
        d_arr.col_len = print_len
        
        return d_arr[::-1]
    
    def topic_oscillations(self, print_len=10, div_fn=KL_div):
        """Returns the oscillation in the divergences of documents
        from each topic k, represented as a categorical distribution
        over topics with mass concentrated at index k. 

        Oscillation is computed as the difference between the maximum
        and the minimum of the divergences.

        Returns an array sorted by descending oscillation.
        """
        topic_indices = np.arange(self.model.K)
            
        pseudo_docs = np.diag(np.ones(self.model.K, dtype='d'))[topic_indices, :]
        rel_entropies = div_fn(pseudo_docs, self.theta)
        oscillations = rel_entropies.max(axis=1) - rel_entropies.min(axis=1)

        k_arr = enum_sort(oscillations).view(LabeledColumn)
        k_arr.col_header = 'Topic Oscillation'
        k_arr.subcol_headers = ['Index', 'Oscillation']
        k_arr.col_len = 10
        return k_arr
    
    def topic_jsds(self, print_len=10):
        """Returns the partial N-way JSD of each topic, where N is the number of
        documents in the model. This measure captures the extent to which an
        individual topic is a reliable signal of a document's overall topic
        distribution.
        
        Returns an array sorted by descending partial JSD.
        """
        topic_indices = np.arange(self.model.K)

        doc_tops = self.theta.T
        M = np.sum(doc_tops, axis=0) / len(doc_tops)
        pjsd = np.sum(np.array([(D_i * np.log(D_i / M)) / len(doc_tops) 
                                    for D_i in doc_tops]), axis=0)

        k_arr = enum_sort(pjsd).view(LabeledColumn)
        k_arr.col_header = 'Topic Partial JSD'
        k_arr.subcol_headers = ['Index', 'pJSD']
        k_arr.col_len = 10
        return k_arr[::-1]

    def _get_sort_header_topic_indices(self, sort=None, topic_indices=None):
        """ 
        Returns a tuple of (str, seq) consisting of the column header and sort
        order for a given sort method.

        :param sort: Topic sort function.
        :type sort: string, values are "entropy", "oscillation", "index", "jsd",
            "user" (default if topic_indices set), "index" (default)
        
        :param topic_indices: List of indices of topics to be
            displayed. Default is all topics.
        :type topic_indices: list of integers
        """
        if sort == 'entropy':
            th = 'Topics Sorted by Entropy'
            ent_sort = self.topic_entropies()['i']
            if topic_indices is not None:
                ti = set(topic_indices)
                topic_indices = [k for k in ent_sort if k in ti]
            else:
                topic_indices = ent_sort
        elif sort == 'oscillation':
            th = 'Topics Sorted by Oscillation'
            osc_sort = self.topic_oscillations()['i']
            if topic_indices is not None:
                ti = set(topic_indices)
                topic_indices = [k for k in osc_sort if k in ti]
            else:
                topic_indices = osc_sort
        elif sort == 'jsd':
            th = 'Topics Sorted by Partial JSD'
            jsd_sort = self.topic_jsds()['i']
            if topic_indices is not None:
                ti = set(topic_indices)
                topic_indices = [k for k in jsd_sort if k in ti]
            else:
                topic_indices = jsd_sort

        elif topic_indices is not None or sort == 'user':
            sort = 'user'
            th = 'Topics Sorted by User'
        else:
            sort = 'index'
            th = 'Topics Sorted by Index' 
            topic_indices = range(self.model.K)

        return (th, np.array(topic_indices))
    
    def topics(self, topic_indices=None, sort=None, print_len=10, 
               as_strings=True, compact_view=True, topic_labels=None):
        """
        Returns a list of topics estimated by the model. 
        Each topic is represented by a list of words and the corresponding 
        probabilities.
        
        :param topic_indices: List of indices of topics to be
            displayed. Default is all topics.
        :type topic_indices: list of integers
        
        :param sort: Topic sort function.
        :type sort: string, values are "entropy", "oscillation", "index", "jsd",
            "user" (default if topic_indices set), "index" (default)

        :param print_len: Number of words shown for each topic. Default is 10.
        :type print_len: int, optional

        :param as_string: If `True`, each topic displays words rather than its
            integer representation. Default is `True`.
        :type as_string: boolean, optional
 
        :param compact_view: If `True`, topics are simply represented as
            their top `print_len` number of words. Otherwise, topics are
            shown as words and their probabilities. Default is `True`.
        :type compact_view: boolean, optional       
    
        :param topic_labels: List of strings that are names that correspond
            to the topics in `topic_indices`.
        :type topic_labels: list, optional

        :returns: an instance of :class:`DataTable`.
            A structured array of topics.
        """
        if topic_indices is not None\
            and (not hasattr(topic_indices, '__len__') or\
                isinstance(topic_indices, (str, unicode))):
            raise ValueError("Invalid value for topic_indices," + 
                             "must be a list of integers")

        th, topic_indices = self._get_sort_header_topic_indices(
                                     sort,topic_indices=topic_indices)

        phi = self.phi[:,topic_indices]

        if as_strings:
	        k_arr = enum_matrix(phi.T, indices=self.corpus.words,
                                    field_name='word')
        else:
            ind = [self.corpus.words_int[word] for word in self.corpus.words]
            k_arr = enum_matrix(phi.T, indices=ind, field_name='word')
       
        table = []
        for i,k in enumerate(topic_indices):
            if topic_labels is None:
                ch = 'Topic ' + str(k)
            else:
                ch = topic_labels[i]

            col = LabeledColumn(k_arr[i], col_header=ch, col_len=print_len)
            table.append(col)

        schc = ['Topic', 'Words']
        schf = ['Word', 'Prob']
        return DataTable(table, th, compact_view=compact_view,
                     subcolhdr_compact=schc, subcolhdr_full=schf)


    def doc_topics(self, doc_or_docs, compact_view=False,
                   aggregate=False, print_len=10, topic_labels=None):
        """
        Returns the distribution over topics for the given documents.

        :param doc: Specifies the document whose distribution over topics is 
             returned. It can either be the ID number (integer) or the 
             name (string) of the document.
        :type doc: int or string
        
        :param print_len: Number of topics to be printed. Default is 10.
        :type print_len: int, optional
        
        :param compact_view: If `True`, topics are simply represented as
        their top `print_len` number of words. Otherwise, topics are
        shown as words and their probabilities. Default is `False`.
        :type compact_view: boolean, optional       

        :param topic_labels: List of strings that are names that correspond
            to the topics in `topic_indices`.
        :type topic_labels: list, optional

        :returns: an instance of :class:`LabeledColumn` or of :class: `DataTable`.
            An structured array of topics (represented by their
            number) and their corresponding probabilities or a list of
            such arrays.
        """
        if (isstr(doc_or_docs) or isint(doc_or_docs) 
            or isinstance(doc_or_docs, dict)):
            doc, label = self._res_doc_type(doc_or_docs)

            k_arr = enum_sort(self.theta[:, doc]).view(LabeledColumn)
            k_arr.col_header = 'Document: ' + label
            k_arr.subcol_headers = ['Topic', 'Prob']
            k_arr.col_len = print_len
            return k_arr

        docs, labels = zip(*[self._res_doc_type(d) for d in doc_or_docs])

        k_arr = enum_matrix(self.theta.T, indices=range(self.model.K), 
                            field_name='topic')

        th = 'Distributions over Topics'
        
        table = []
        for i in xrange(len(docs)):
            if topic_labels is None: 
                ch = 'Doc: ' + labels[i]
            else:
                ch = topic_labels[i] 
                
            col = LabeledColumn(k_arr[docs[i]], col_header=ch, col_len=print_len)
            table.append(col)

        schc = ['Doc', 'Topics']
        schf = ['Topic', 'Prob']
        return DataTable(table, th, compact_view=compact_view,
                        subcolhdr_compact=schc, subcolhdr_full=schf)


    def aggregate_doc_topics(self, docs, normed_sum=False, print_len=10):
        """Takes a list of documents identifiers and returns the sum of the
        distributions over topics corresponding to these topics,
        normalized to sum to 1. If normed_sum is True, the sum is
        weighted by document lengths, so that documents contribute
        uniformly to the aggregate distribution.

        """
        docs, labels = zip(*[self._res_doc_type(d) for d in docs])
        
        if normed_sum:
            S = self.theta[:, docs].sum(1)
        else:
            S = self.model.top_doc[:, docs].sum(1)

        S = S / S.sum()

        k_arr = enum_sort(S).view(LabeledColumn)
        k_arr.col_header = 'Aggregate Distribution over Topics'
        k_arr.subcol_headers = ['Topic', 'Prob']
        k_arr.col_len = print_len

        return k_arr
    
    def doc_topic_matrix(self, doc_or_docs):
        """
        Returns the distribution over topics for the given documents.

        :param doc: Specifies the document whose distribution over topics is 
             returned. It can either be the ID number (integer) or the 
             name (string) of the document.
        :type doc: int or string

        :returns: an instance of :class:`np.array`.
            An array of topics
        """
        if (isstr(doc_or_docs) or isint(doc_or_docs) 
            or isinstance(doc_or_docs, dict)):
            doc, label = self._res_doc_type(doc_or_docs)
            k_arr = self.theta[:, doc].T
        else:
            docs, labels = zip(*[self._res_doc_type(d) for d in doc_or_docs])
            k_arr = self.theta[:, docs].T

        return k_arr
    
    def view_documents(self, doc_or_docs, as_strings=False):
        """
        Returns the distribution over topics for the given documents.

        :param doc: Specifies the document whose distribution over topics is 
             returned. It can either be the ID number (integer) or the 
             name (string) of the document.
        :type doc: int or string

        :returns: an instance of :class:`np.array`.
            An array of topics
        """
        documents = []
        if (isstr(doc_or_docs) or isint(doc_or_docs) 
            or isinstance(doc_or_docs, dict)):
            doc, label = self._res_doc_type(doc_or_docs)
            docs = list(doc)
        else:
            docs, labels = zip(*[self._res_doc_type(d) for d in doc_or_docs])
        
        all_docs = self.corpus.view_contexts(self.model.context_type,
                        as_strings=as_strings)
        for doc in docs:
            documents.append(all_docs[doc])

        return documents

    def word_topics(self, word, as_strings=True):
        """
        Searches for every occurrence of `word` in the entire corpus and returns 
        a list each row of which contains the name or ID number of document, 
        the relative position in the document, and the assigned topic number 
        for each occurrence of `word`.
        
        :param word: The word for which the search is performed.  
        :type word: string

        :param as_strings: If `True`, returns document names rather than 
            ID numbers. Default is `True`.
        :type as_strings: boolean, optional

        :returns: an instance of :class:`LabeledColumn`.
            A structured array consisting of three columns. Each column 
            is a list of:
            (1) name/ID of document containing `word`
            (2) relative position of `word` in the document
            (3) Topic number assigned to the token.
        """
        w, word = self._res_word_type(word)

        # Search for occurrences of a word in the corpus and return a
        # positions and topic assignments for each found
        ct = self.model.context_type
        contexts = self.corpus.view_contexts(ct)
        idx = [(contexts[d] == w) for d in xrange(len(contexts))]
        Z = split_corpus(self.model.Z, self.model.indices)
        Z_w = [(d, i, t) for d in xrange(len(Z)) 
               for i,t in enumerate(Z[d]) if idx[d][i]]

        # Label data
        if as_strings:
            docs = self.corpus.view_metadata(ct)[self._doc_label_name]
            dt = [('doc', docs.dtype), ('pos',np.int), ('value', np.int)]
            Z_w = [(docs[d], i, t) for (d, i, t) in Z_w]
        else:
            dt = [('i', np.int), ('pos',np.int), ('value', np.int)]

        Z_w = np.array(Z_w, dtype=dt).view(LabeledColumn)
        Z_w.col_header = 'Word: ' + word
        Z_w.subcol_headers = ['Document', 'Pos', 'Topic']

        return Z_w


    @deprecated_meth("dist_top_top")
    def sim_top_top(self, topic_or_topics, weights=[], 
                     dist_fn=JS_dist, order='i', 
                     show_topics=True, print_len=10, filter_nan=True, 
                     as_strings=True, compact_view=True):
        pass      

    def dist_top_top(self, topic_or_topics, weights=[], 
                     dist_fn=JS_dist, order='i', 
                     show_topics=True, print_len=10, filter_nan=True, 
                     as_strings=True, compact_view=True, topic_labels=None):
        """
        Takes a topic or list of topics (by integer index) and returns
        a list of topics sorted by the distances between a given
        topic and every topic.
        
        :param topic_or_topics: Query topic(s) to which distances are calculated.
        :type topic_or_topics: integer or list of integers
        
        :param weights: Specify weights for each topic in `topic_or_topics`. 
            Default uses equal weights (i.e. arithmetic mean)
        :type weights: list of floating point, optional

        :param show_topics: If `True`, topics are represented by their number
            and distribution over words. Otherwise only topic numbers
            are shown. Default is `True`.
        :type show_topics: boolean, optional

        :param print_len: Number of topics to be shown. Default is 10.
        :type print_len: int, optional       

        :param filter_nan: If `True` not a number entries are filtered.
            Default is `True`.
        :type filter_nan: boolean, optional
        
        :param dist_fn: A distance function from functions in vsm.spatial. 
            Default is :meth:`JS_dist`.
        :type dist_fn: string, optional

        :param order: Order of sorting. 'i' for increasing and 'd' for
            decreasing order. Default is 'i'.
        :type order: string, optional

        :param as_strings: If `True`, words of each topic are represented as
            strings. Otherwise they are represented by their integer
            representation. Default is `True`.
        :type as_strings: boolean, optional

        :param compact_view: If `True`, topics are simply represented as
            their top `print_len` number of words. Otherwise, topics are
            shown as words and their probabilities. Default is `True`.
        :type compact_view: boolean, optional       
        
        :param topic_labels: List of strings that are names that correspond
            to the topics in `topic_indices`.
        :type topic_labels: list, optional

        :returns: an instance of :class:`LabeledColumn`.
            A 2-dim array containing topics and their distances to 
            `topic_or_topics`. 
        
        :See Also: :meth:`vsm.viewer.wrapper.dist_top_top`
        """
        Q = self.model.word_top / self.model.word_top.sum(0)

        distances = dist_top_top(Q, topic_or_topics, weights=weights, 
                                 print_len=print_len, filter_nan=filter_nan, 
                                 dist_fn=dist_fn, order=order)

        if show_topics:
            topic_or_topics = [topic_or_topics]
            topic_indices = distances[distances.dtype.names[0]]
        
            k_arr = self.topics(topic_indices=topic_indices, print_len=print_len,
                                as_strings=as_strings, compact_view=compact_view,
                                topic_labels=topic_labels)
            
            k_arr.table_header = 'Sorted by Topic Distance'
            for i in xrange(distances.size):
                k_arr[i].col_header += ' ({0:.3f})'.format(distances[i][1])

            return k_arr

        return distances 

    
    @deprecated_meth("dist_top_doc")
    def sim_top_doc(self, topic_or_topics, weights=[], filter_words=[],
                     print_len=10, as_strings=True, label_fn=def_label_fn, 
                     filter_nan=True, dist_fn=JS_dist, order='i'):
        pass

    def dist_top_doc(self, topic_or_topics, weights=[], filter_words=[],
                     print_len=10, as_strings=True, label_fn=def_label_fn, 
                     filter_nan=True, dist_fn=JS_dist, order='i'):
        """Takes a topic or list of topics (by integer index) and returns
        a list of documents sorted by distance.

        :param topic_or_topics: Query topic(s) relative to which
            distances are computed.
        :type topic_or_topics: string or list of strings
        
        :param weights: Specify weights for each topic in `topic_or_topics`. 
            Default uses equal weights (i.e. arithmetic mean)
        :type weights: list of floating point, optional

        :param filter_words: The topics that include these words are considered.
            If not provided, by default all topics are considered.
        :type filter_words: list of words, optional
 
        :param print_len: Number of documents printed by pretty-pringing function
            Default is 10.
        :type print_len: int, optional       
        
        :param as_strings: If `True`, returns a list of documents as strings rather
            than their integer representations. Default is `True`.
        :type as_strings: boolean, optional

        :param label_fn: A function that defines how documents are represented.
            Default is def_label_fn which retrieves the labels from corpus
            metadata.
        :type label_fn: string, optional

        :param filter_nan: If `True` not a number entries are filtered.
            Default is `True`.
        :type filter_nan: boolean, optional
        
        :param dist_fn: A distance function from functions in vsm.spatial. 
            Default is :meth:`JS_dist`.
        :type dist_fn: string, optional

        :param order: Order of sorting. 'i' for increasing and 'd' for
            decreasing order. Default is 'i'.
        :type order: string, optional

        :returns: an instance of :class:`LabeledColumn`.
            A 2-dim array containing documents and their posterior probabilities 
            to `topic_or_topics`. 

        :See Also: :meth:`def_label_fn`, :meth:`vsm.viewer.wrapper.dist_top_doc`

        """
        Q = self.model.top_doc / self.model.top_doc.sum(0)

        d_arr = dist_top_doc(topic_or_topics, Q, self.corpus,   
                               self.model.context_type, weights=weights, 
                               print_len=print_len, as_strings=False, 
                               label_fn=label_fn, filter_nan=filter_nan, 
                               dist_fn=dist_fn)

        topics = res_top_type(topic_or_topics)

        if len(filter_words) > 0:
            white = set()
            for w in filter_words:
                l = self.word_topics(w, as_strings=False)
                d = l['i'][np.in1d(l['value'], topics)]
                white.update(d)
            
            d_arr = d_arr[(np.in1d(d_arr['i'], white))]

        if as_strings:
            md = self.corpus.view_metadata(self.model.context_type)
            docs = label_fn(md)
            d_arr = map_strarr(d_arr, docs, k='i', new_k='doc')

    	return d_arr

   
    @deprecated_meth("dist_word_top")
    def sim_word_top(self, word_or_words, weights=[], filter_nan=True,
                      show_topics=True, print_len=10, as_strings=True, 
                      compact_view=True, dist_fn=JS_dist, order='i'):
        pass
   
    def dist_word_top(self, word_or_words, weights=[], filter_nan=True,
                      show_topics=True, print_len=10, as_strings=True, 
                      compact_view=True, dist_fn=JS_dist, order='i',
                      topic_labels=None):
        """Sorts topics according to their distance to the query
        `word_or_words`.
        
        A pseudo-topic from `word_or_words` as follows. If weights are
        not provided, the word list is represented in the space of
        topics as a topic which assigns equal non-zero probability to
        each word in `words` and 0 to every other word in the
        corpus. Otherwise, each word in `words` is assigned the
        provided weight.
        
        :param word_or_words: word(s) to which distances are calculated
        :type word_or_words: string or list of strings
        
        :param weights: Specify weights for each query word in `word_or_words`. 
            Default uses equal weights.
        :type weights: list of floating point, optional
        
        :param filter_nan: If `True` not a number entries are filtered.
            Default is `True`.
        :type filter_nan: boolean, optional
            
        :param show_topics: If `True`, topics are represented by their number
            and distribution over words. Otherwise, only topic numbers
            are shown. Default is `True`. 
        :type show_topics: boolean, optional

        :param print_len: Number of words printed by pretty-printing function.
            Default is 10.
        :type print_len: int, optional

        :param as_strings: If `True`, words of each topic are represented as
            strings. Otherwise they are represented by their integer 
            representation. Default is `True`.
        :type as_strings: boolean, optional

        :param compact_view: If `True`, topics are simply represented as
            their top `print_len` number of words. Otherwise, topics are
            shown as words and their probabilities. Default is `True`.
        :type compact_view: boolean, optional       
        
        :param dist_fn: A distance function from functions in vsm.spatial. 
            Default is :meth:`JS_dist`.
        :type dist_fn: string, optional

        :param order: Order of sorting. 'i' for increasing and 'd' for
            decreasing order. Default is 'i'.
        :type order: string, optional
        
        :param topic_labels: List of strings that are names that correspond
            to the topics in `topic_indices`.
        :type topic_labels: list, optional

        :returns: an instance of :class:`LabeledColumn`.
            A structured array of topics sorted by their distances 
            with `word_or_words`.
        
        :See Also: :meth:`vsm.viewer.wrapper.dist_word_top`

        """
        Q = self.model.word_top / self.model.word_top.sum(0)

        distances = dist_word_top(word_or_words, self.corpus, Q,  
                                  weights=weights, print_len=print_len, 
                                  filter_nan=filter_nan,
                                  dist_fn=dist_fn, order=order)

        if show_topics:
            if isstr(word_or_words):
                word_or_words = [word_or_words]

            # Filter based on topic assignments to words (Z values) 
            topic_indices = sum([self.word_topics(w)['value'].tolist() 
                           for w in word_or_words], [])
            topic_indices = [i for i in xrange(distances.size) if distances[i][0] in topic_indices]
            distances = distances[topic_indices]
            topic_indices = distances[distances.dtype.names[0]]

            k_arr = self.topics(topic_indices=topic_indices, print_len=print_len,
                                as_strings=as_strings, compact_view=compact_view,
                                topic_labels=topic_labels)

            # Retrieve topics
            if compact_view:
                k_arr.table_header = 'Sorted by Topic Distance'
                return k_arr

            # Relabel results
            k_arr.table_header = 'Sorted by Topic Distance'
            for i in xrange(distances.size):
                k_arr[i].col_header += ' ({0:.5f})'.format(distances[i][1])

            return k_arr

        return distances


    @deprecated_meth("dist_doc_doc")
    def sim_doc_doc(self, doc_or_docs, 
                     print_len=10, filter_nan=True, 
                     label_fn=def_label_fn, as_strings=True,
                     dist_fn=JS_dist, order='i'):
        pass

    def dist_doc_doc(self, doc_or_docs, 
                     print_len=10, filter_nan=True, 
                     label_fn=def_label_fn, as_strings=True,
                     dist_fn=JS_dist, order='i'):
        """Computes and sorts the distances between a document 
        or list of documents and every document in the topic space.
 
        :param doc_or_docs: Query document(s) relative to which
        distances are computed.
        :type doc_or_docs: string/integer or list of strings/integer.

        :param print_len: Number of words printed by pretty-printing function.
            Default is 10.
        :type print_len: int, optional

        :param filter_nan: If `True` not a number entries are filtered.
            Default is `True`.
        :type filter_nan: boolean, optional

        :param label_fn: A function that defines how documents are represented.
            Default is def_label_fn which retrieves the labels from corpus
            metadata.
        :type label_fn: string, optional

        :param as_strings: If `True`, returns a list of words rather than
            their integer representations. Default is `True`.
        :type as_strings: boolean, optional
        
        :param dist_fn: A distance function from functions in vsm.spatial. 
            Default is :meth:`JS_dist`.
        :type dist_fn: string, optional

        :param order: Order of sorting. 'i' for increasing and 'd' for
            decreasing order. Default is 'i'.
        :type order: string, optional
       
        :returns: an instance of `LabeledColumn`.
            A 2-dim array containing documents and their distances to 
            `doc_or_docs`. 
        
        :See Also: :meth:`vsm.viewer.wrapper.dist_doc_doc`

        """
        Q = self.model.top_doc / self.model.top_doc.sum(0)

        return dist_doc_doc(doc_or_docs, self.corpus, 
                            self.model.context_type, Q,  
                            print_len=print_len, filter_nan=filter_nan, 
                            label_fn=label_fn, as_strings=as_strings,
                            dist_fn=dist_fn, order=order)

    @deprecated_meth("dismat_doc")
    def simmat_docs(self, docs=[], dist_fn=JS_dist):
        pass
    
    def dismat_doc(self, docs=[], dist_fn=JS_dist):
        """
        Calculates the distance matrix for a given list of documents.

        :param docs: A list of documents whose distance matrix is to be computed.
            Default is all the documents in the model.
        :type docs: list, optional
        
        :param dist_fn: A distance function from functions in vsm.spatial. 
            Default is :meth:`JS_dist`.
        :type dist_fn: string, optional

        :returns: an instance of :class:`IndexedSymmArray`.
            n x n matrix containing floats where n is the number of documents.

        :See Also: :meth:`vsm.viewer.wrapper.dismat_documents`
        """
        if len(docs) == 0:
            docs = range(self.model.top_doc.shape[1])

        Q = self.model.top_doc / self.model.top_doc.sum(0)

        dm =  dismat_doc(docs, self.corpus, self.model.context_type,
                           Q, dist_fn=dist_fn)

        return dm


    @deprecated_meth("dismat_top") 
    def simmat_topics(self, topics=[], dist_fn=JS_dist):
        pass

    def dismat_top(self, topics=[], dist_fn=JS_dist):
        """
        Calculates the distance matrix for a given list of topics.

        :param topic_indices: A list of topics whose distance matrix is to be
            computed. Default is all topics in the model.
        :type topic_indices: list, optional
        
        :param dist_fn: A distance function from functions in vsm.spatial. 
            Default is :meth:`JS_dist`.
        :type dist_fn: string, optional

        :returns: an instance of :class:`IndexedSymmArray`.
            n x n matrix containing floats where n is the number of
            topics considered.
        
        :See Also: :meth:`vsm.viewer.wrapper.dismat_top`
        """

        if len(topics) == 0:
            topics = range(self.model.word_top.shape[1])

        Q = self.model.word_top / self.model.word_top.sum(0)

        return dismat_top(topics, Q, dist_fn=dist_fn)

    def dist(self, doc1, doc2, dist_fn=JS_dist):
        """Computes the distance between 2 documents in topic space.
 
        :param doc1: Query document        
        :type doc1: string/integer
 
        :param doc2: Query document        
        :type doc2: string/integer

        :param dist_fn: A distance function from functions in vsm.spatial. 
            Default is :meth:`JS_dist`.
        :type dist_fn: string, optional
       
        :returns: an instance of `LabeledColumn`.
            A 2-dim array containing documents and their distances to 
            `doc_or_docs`. 
        """
        d1, d2 = self.doc_topic_matrix([doc1,doc2])
        return dist_fn(d1,d2)
        
        

    ######################################################################


    #TODO: Move to plotting extension
    def logp_plot(self, range=[], step=1, show=True, grid=True):
        """
        Returns a plot of log probabilities for the specified range of 
        the MCMC chain used to fit a topic model by `LDAGibbs`.
        The function requires matplotlib package. 

        :param range: Specifies the range of the MCMC chain whose log probabilites 
            are to be plotted. For example, range = [0, 999] plots log 
            probabilities from the 1st to the 1000th iterations. 
            The length of the list must be exactly two, and the first 
            element must be smaller than the second which can not exceed 
            the total length of the MCMC chain.
            Default produces the plot for the entire chain.
        :type range: list of integers, optional
        
        :param step: Steps by which points are plotted. Default is 1.
        :type step: int, optional

        :param show: If `True`, the function actually draws the plot 
            in addition to returning a plot object. Default is `True`.
        :type show: boolean, optional

        :param grid: If `True` draw a grid. Default is `True`. 
        :type grid: boolean, optional
        
        :returns: an instance of matplotlib.pyplot object.
            Contains the log probability plot. 
        """
        import matplotlib.pyplot as plt

        # If range is not specified, include the whole chain.
        if len(range) == 0:
            range = [0, len(self.model.log_probs)]

        x = []
        logp = []
        for i, lp in self.model.log_probs[range[0]:range[1]:step]:
            x.append(i)
            logp.append(lp)

        plt.plot(x,logp)
        plt.xlim(min(x), max(x))
        plt.grid(grid)
        plt.title('log probability / iteration')

        if show:
            plt.show()

        return plt


    # TODO: Move to plotting extension
    def topic_hist(self, topic_indices=None, d_indices=[], show=True):
        """
        Draws a histogram showing the proportion of topics within a set of
        documents specified by d_indices. 

        :param topic_indices: Specifies the topics for which proportions are 
             calculated. Default is all topics.
        :type doc: list of integers, optional
        
        :param d_indices: Specifies the document for which topic proportions
             are calculated. Default is all documents.
        :type d_indices: list of integers, optional

        :param show: shows plot if `True`. Default is `True`.
        :type d_indices: boolean, optional
        
        :returns: an instance of matplotlib.pyplot object.
             Contains the topic proportion histogram.
        """
        import matplotlib.pyplot as plt

        if len(d_indices) == 0:
            i = self.corpus.context_types.index(self.model.context_type)
            d_indices = xrange(len(self.corpus.context_data[i]))

        td_mat = self.model.top_doc[:, d_indices]
        td_mat /= td_mat.sum(0)
        arr = td_mat.sum(1)

        if not topic_indices == None:
            arr = arr[topic_indices]

        l = enum_sort(arr)
        rank, prob = zip(*l)

        y_pos = np.arange(len(rank))
        fig = plt.figure(figsize=(10,10))

        plt.barh(y_pos, list(prob))
        plt.yticks(y_pos, rank)
        plt.xlabel('Frequencies')
        plt.title('Topic Proportions')

        if show:
            plt.show()

        return plt





from vsm.spatial import angle
from vsm.exceptions import *
from wrappers import *


__all__ = ['TfIdfViewer']


class TfIdfViewer(object):
    """
    A class for viewing Term frequency-Inverse document Frequency model.
    """
    
    def __init__(self, corpus, model):
        """
        Initialize TfIdfViewer.
        
        :param corpus: Source of observed data.
        :type corpus: :class:`Corpus`

        :param model: A Term frequency-Inverse document Frequency model.
        :type model: TfIdf.
        """
        self.corpus = corpus
        self.model = model


    @deprecated_meth("dist_word_word")    
    def sim_word_word(self, word_or_words, weights=[], 
                       filter_nan=True, print_len=10, as_strings=True, 
                       dist_fn=angle, order='i'):
        pass

    def dist_word_word(self, word_or_words, weights=[], 
                       filter_nan=True, print_len=10, as_strings=True, 
                       dist_fn=angle, order='i'):
        """
        Computes and sorts the distances between a word or list
        of words and every word.

        :param word_or_words: Query word(s) to which distances are calculated.
        :type word_or_words: string or list of strings
        
        :param weights: Specify weights for each query word in `word_or_words`. 
            Default uses equal weights (i.e. arithmetic mean)
        :type weights: list of floating point, optional
        
        :param filter_nan: If `True` not a number entries are filtered.
            Default is `True`.
        :type filter_nan: boolean, optional

        :param print_len: Number of words to be displayed. Default is 10.
        :type print_len: int, optional

        :param as_strings: If `True`, returns a list of words as strings rather
            than their integer representations. Default is `True`.
        :type as_strings: boolean, optional
               
        :param dist_fn: A distance function from functions in vsm.spatial. 
            Default is :meth:`angle`.
        :type dist_fn: string, optional
        
        :param order: Order of sorting. 'i' for increasing and 'd' for
            decreasing order. Default is 'i'.
        :type order: string, optional

        :returns: an instance of :class:`LabeledColumn`.
            A 2-dim array containing words and their cosine values to 
            `word_or_words`.

        :See Also: :meth:`vsm.viewer.wrappers.dist_word_word`
        """
        return dist_word_word(word_or_words, self.corpus, 
                                self.model.matrix.T.toarray(), weights=weights, 
                                filter_nan=filter_nan, 
                                print_len=print_len, as_strings=True,
                                dist_fn=dist_fn, order=order)

   
    @deprecated_meth("dist_doc_doc")
    def sim_doc_doc(self, doc_or_docs, weights=[], print_len=10, 
                     filter_nan=True, label_fn=def_label_fn, as_strings=True,
                     dist_fn=angle, order='i'):
        pass

    def dist_doc_doc(self, doc_or_docs, weights=[], print_len=10, 
                     filter_nan=True, label_fn=def_label_fn, as_strings=True,
                     dist_fn=angle, order='i'):
        """
        Computes and sorts the distances between a document or list
        of documents and every document.

        :param doc_or_docs: Query document(s) to which distances
            are calculated.
        :type doc_or_docs: string/integer or list of strings/integers
        
        :param weights: Specify weights for each query doc in `doc_or_docs`. 
            Default uses equal weights (i.e. arithmetic mean)
        :type weights: list of floating point, optional

        :param print_len: Number of words to be displayed. Default is 10.
        :type print_len: int, optional

        :param filter_nan: If `True` not a number entries are filtered.
            Default is `True`.
        :type filter_nan: boolean, optional
 
        :param label_fn: A function that defines how documents are represented.
            Default is def_label_fn which retrieves the labels from corpus
            metadata.
        :type label_fn: string, optional
        
        :param as_strings: If `True`, returns a list of words rather than
            their integer representations. Default is `True`.
        :type as_strings: boolean, optional
        
        :param dist_fn: A distance function from functions in vsm.spatial. 
            Default is :meth:`angle`.
        :type dist_fn: string, optional
        
        :param order: Order of sorting. 'i' for increasing and 'd' for
            decreasing order. Default is 'i'.
        :type order: string, optional

        :returns: an instance of :class:`LabeledColumn`.
            A 2-dim array containing documents and their cosine values to 
            `doc_or_docs`. 

        :See Also: :meth:`vsm.viewer.wrappers.dist_doc_doc`
        """
        return dist_doc_doc(doc_or_docs, self.corpus, self.model.context_type, 
                              self.model.matrix.toarray(), weights=weights,
                              print_len=print_len, filter_nan=filter_nan, 
                              label_fn=def_label_fn, as_strings=True,
                              dist_fn=dist_fn, order=order)

    @deprecated_meth("dist_word_doc")
    def sim_word_doc(self, word_or_words, weights=[], label_fn=def_label_fn, 
                      filter_nan=True, print_len=10, as_strings=True, 
                      dist_fn=angle, order='i'):
        pass

    def dist_word_doc(self, word_or_words, weights=[], label_fn=def_label_fn, 
                      filter_nan=True, print_len=10, as_strings=True, 
                      dist_fn=angle, order='i'):
        """
        Computes and sorts distances between a word or a list of words to every
        document.
        
        :param word_or_words: Query word(s) to which a pseudo-document is
            created for computation of distances.
        :type word_or_words: string/integer or list of strings/integers
        
        :param weights: Specify weights for each query doc in `word_or_words`. 
            Default uses equal weights (i.e. arithmetic mean)
        :type weights: list of floating point, optional
        
        :param print_len: Number of documents to be displayed. Default is 10.
        :type print_len: int, optional

        :param filter_nan: If `True` not a number entries are filtered.
            Default is `True`.
        :type filter_nan: boolean, optional
 
        :param label_fn: A function that defines how documents are represented.
            Default is :meth:`def_label_fn` which retrieves the labels 
            from corpus metadata.
        :type label_fn: string, optional
        
        :param as_strings: If `True`, returns a list of documents as strings
            rather than indices. Default is `True`.
        :type as_strings: boolean, optional

        :param dist_fn: A distance function from functions in vsm.spatial.
            Default is :meth:`angle`.
        :type dist_fn: string, optional
        
        :param order: Order of sorting 'i' for increasing and 'd' for
            decreasing order. Default is 'i'.
        :type order: string, optional

        :returns: an instance of :class:`LabeledColumn`.
            A 2-dim array containing documents and their distances to 
            `word_or_words`. 

        :See Also: :meth:`vsm.viewer.wrappers.dist_word_doc`
        """
        return dist_word_doc(word_or_words, self.corpus, 
                               self.model.context_type, 
                               self.model.matrix.toarray(), weights=weights, 
                               label_fn=label_fn,
                               filter_nan=filter_nan, 
                               print_len=print_len, as_strings=as_strings,
                               dist_fn=dist_fn, order=order)

    
    @deprecated_meth("dismat_word")
    def simmat_words(self, word_list, dist_fn=angle):
        pass
    
    def dismat_word(self, word_list, dist_fn=angle):
        """
        Calculates a distance matrix for a given list of words.

        :param word_list: A list of words whose distance matrix is to be
            computed.
        :type word_list: list
        
        :param dist_fn: A distance function from functions in vsm.spatial. 
            Default is :meth:`angle`.
        :type dist_fn: string, optional

        :returns: an instance of :class:`IndexedSymmArray`. 
            n x n matrix containing floats where n is the number 
            of words in `word_list`.

        :See Also: :meth:`vsm.viewer.wrappers.distmat_word`
        """
        return dismat_word(word_list, self.corpus,
                             self.model.matrix.T.toarray(), dist_fn=dist_fn)


    @deprecated_meth("dismat_doc")
    def simmat_docs(self, doc_list, dist_fn=angle):
        pass

    def dismat_doc(self, doc_list, dist_fn=angle):
        """
        Calculates a distance matrix for a given list of documents.

        :param doc_list: A list of documents whose distance matrix is 
            to be computed.
        :type doc_list: list
        
        :param dist_fn: A distance function from functions in vsm.spatial. 
            Default is :meth:`angle`.
        :type dist_fn: string, optional

        :returns: an instance of :class:`IndexedSymmArray`.
            n x n matrix containing floats where n is the number of 
            documents.

        :See Also: :meth:`vsm.viewer.wrappers.dismat_doc`
        """
        return dismat_doc(doc_list, self.corpus, self.model.context_type, 
                            self.model.matrix.toarray(), dist_fn=dist_fn)
